@@ -57,6 +57,7 @@ elif [ "${TARGET}" = "linux-strict" ]; then
 elif [ "${TARGET}" = "linux-juce-strict" ]; then
     sudo apt-get install -y \
         g++-8 \
+        g++-8-multilib \
         pkg-config \
         pyqt5-dev-tools \
         python3-pyqt5.qtsvg \
@@ -72,25 +73,22 @@ elif [ "${TARGET}" = "linux-juce-strict" ]; then
         libgl1-mesa-dev \
         libx11-dev \
         libxext-dev \
+        libx11-6:i386 \
+        libxext6:i386 \
+        libfreetype6:i386 \
         liblo-static \
         fluidsynth-static
 
+    # Fix for 32bit bridge links
+    sudo ln -s /usr/lib/i386-linux-gnu/libX11.so.6 /usr/lib/i386-linux-gnu/libX11.so
+    sudo ln -s /usr/lib/i386-linux-gnu/libXext.so.6 /usr/lib/i386-linux-gnu/libXext.so
+    sudo ln -s /usr/lib/i386-linux-gnu/libfreetype.so.6 /usr/lib/i386-linux-gnu/libfreetype.so
+
 elif [ "${TARGET}" = "macos" ]; then
     sudo apt-get install -y \
         pkg-config \
         apple-x86-setup
 
-# mkdir /tmp/osx-macports-pkgs
-# cd /tmp/osx-macports-pkgs
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-flac_1.2.1-1_all.deb
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-libiconv_1.14-0_all.deb
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-liblo_0.26-1_all.deb
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-libogg_1.3.0-1_all.deb
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-libsndfile_1.0.25-0_all.deb
-# wget https://launchpad.net/~kxstudio-team/+archive/ubuntu/builds/+files/apple-macports-libvorbis_1.3.3-0_all.deb
-# cd ~
-# rm -r /tmp/osx-macports-pkgs
-
 elif [ "${TARGET}" = "win32" ]; then
     sudo apt-get install -y \
         mingw32-x-gcc \
@@ -103,15 +101,12 @@ elif [ "${TARGET}" = "win64" ]; then
         mingw64-x-gcc \
         mingw64-x-pkgconfig
 
-fi
+elif [ "${TARGET}" = "pylint" ]; then
+    sudo apt-get install -y \
+        pylint3 \
+        python3-liblo \
+        python3-pyqt5 python3-pyqt5.qtsvg python3-pyqt5.qtopengl python3-rdflib \
+        pyqt5-dev-tools \
+        qtbase5-dev
 
-#     mingw32-x-fluidsynth
-#     mingw32-x-fftw
-#     mingw32-x-liblo
-#     mingw32-x-mxml
-#     mingw32-x-zlib
-#     mingw64-x-fluidsynth
-#     mingw64-x-fftw
-#     mingw64-x-liblo
-#     mingw64-x-mxml
-#     mingw64-x-zlib
+fi
