@@ -337,6 +337,11 @@ fi
 
 # cleanup
 
+if [ $(clang -v  2>&1 | grep version | cut -d' ' -f4 | cut -d'.' -f1) -lt 11 ]; then
+  export ARCH=32
+  build_base
+fi
+
 export ARCH=64
 build_base
 
@@ -458,7 +463,7 @@ fi
 # sip
 
 if [ ! -d sip-${SIP_VERSION} ]; then
-  /opt/local/bin/aria2c https://www.riverbankcomputing.com/static/Downloads/sip/${SIP_VERSION}/sip-${SIP_VERSION}.tar.gz
+  /opt/local/bin/aria2c https://files.kde.org/krita/build/dependencies/sip-${SIP_VERSION}.tar.gz
   tar -xf sip-${SIP_VERSION}.tar.gz
 fi
 
@@ -475,7 +480,7 @@ fi
 # pyqt5
 
 if [ ! -d PyQt5_gpl-${PYQT5_VERSION} ]; then
-  /opt/local/bin/aria2c https://www.riverbankcomputing.com/static/Downloads/PyQt5/${PYQT5_VERSION}/PyQt5_gpl-${PYQT5_VERSION}.tar.gz
+  /opt/local/bin/aria2c https://files.kde.org/krita/build/dependencies/PyQt5_gpl-${PYQT5_VERSION}.tar.gz
   tar -xf PyQt5_gpl-${PYQT5_VERSION}.tar.gz
 fi
 
