@@ -14,6 +14,8 @@ from carla_host import VERSION
 
 # ------------------------------------------------------------------------------------------------------------
 
+SCRIPT_NAME = getenv("SCRIPT_NAME")
+
 options = {
   "zip_include_packages": ["*"],
   "zip_exclude_packages": ["PyQt5"],
@@ -25,10 +27,13 @@ boptions = {
   "iconfile": "./resources/ico/carla.icns"
 }
 
+if SCRIPT_NAME in ("Carla", "Carla-Control"):
+  boptions["custom_info_plist"] = "./data/macos/%s_Info.plist" % SCRIPT_NAME
+
 setup(name = "Carla",
       version = VERSION,
       description = "Carla Plugin Host",
       options = {"build_exe": options, "bdist_mac": boptions},
-      executables = [Executable("./source/frontend/%s.pyw" % getenv("SCRIPT_NAME"))])
+      executables = [Executable("./source/frontend/%s.pyw" % SCRIPT_NAME)])
 
 # ------------------------------------------------------------------------------------------------------------
