@@ -1,6 +1,6 @@
 #!/bin/bash
 
-VERSION="2.1.1"
+VERSION="2.2.0"
 
 # ---------------------------------------------------------------------------------------------------------------------
 # stop on error
@@ -14,6 +14,19 @@ if [ ! -f Makefile ]; then
   cd $(dirname $0)/../..
 fi
 
+# ---------------------------------------------------------------------------------------------------------------------
+# cleanup
+
+if [ ! -f Makefile ] || [ ! -f source/frontend/carla ]; then
+    echo "wrong dir"
+    exit 1
+fi
+
+rm -rf data/windows/*.exe
+rm -rf data/windows/*.lv2
+rm -rf data/windows/*.vst
+rm -rf data/windows/*.zip
+
 # ---------------------------------------------------------------------------------------------------------------------
 # start
 
