@@ -33,7 +33,7 @@ if [ x"${ARCH}" = x"32nosse" ]; then
   MAKE_ARGS="${MAKE_ARGS} NOOPT=true"
 fi
 
-MAKE_ARGS="${MAKE_ARGS} BUILDING_FOR_WINDOWS=true USING_JUCE=true"
+MAKE_ARGS="${MAKE_ARGS} BUILDING_FOR_WINDOWS=true CROSS_COMPILING=true"
 
 export WIN32=true
 
@@ -72,6 +72,7 @@ export LDFLAGS="-L${DEPS_PREFIX}/lib"
 export MOC_QT5="wine ${MSYS2_PREFIX}/bin/moc.exe"
 export RCC_QT5="wine ${MSYS2_PREFIX}/bin/rcc.exe"
 export UIC_QT5="wine ${MSYS2_PREFIX}/bin/uic.exe"
+
 }
 
 # ---------------------------------------------------------------------------------------------------------------------
