@@ -54,12 +54,12 @@ CARLA_BACKEND_START_NAMESPACE
 /*!
  * Maximum default number of loadable plugins.
  */
-static const uint MAX_DEFAULT_PLUGINS = 99;
+static const uint MAX_DEFAULT_PLUGINS = 512;
 
 /*!
  * Maximum number of loadable plugins in rack mode.
  */
-static const uint MAX_RACK_PLUGINS = 16;
+static const uint MAX_RACK_PLUGINS = 64;
 
 /*!
  * Maximum number of loadable plugins in patchbay mode.
@@ -188,6 +188,13 @@ static const uint PLUGIN_USES_MULTI_PROGS = 0x400;
  */
 static const uint PLUGIN_HAS_INLINE_DISPLAY = 0x800;
 
+/*!
+ * Plugin has its own custom UI which can be embed into another Window.
+ * @see CarlaPlugin::embedCustomUI() and carla_embed_custom_ui()
+ * @note This is very experimental and subject to change at this point
+ */
+static const uint PLUGIN_HAS_CUSTOM_EMBED_UI = 0x1000;
+
 /** @} */
 
 /* ------------------------------------------------------------------------------------------------------------
@@ -253,6 +260,13 @@ static const uint PLUGIN_OPTION_SEND_ALL_SOUND_OFF = 0x100;
  */
 static const uint PLUGIN_OPTION_SEND_PROGRAM_CHANGES = 0x200;
 
+/*!
+ * Skip sending MIDI note events.
+ * This if off-by-default as a way to keep backwards compatibility.
+ * We always want notes enabled by default, not the contrary.
+ */
+static const uint PLUGIN_OPTION_SKIP_SENDING_NOTES = 0x400;
+
 /*!
  * Special flag to indicate that plugin options are not yet set.
  * This flag exists because 0x0 as an option value is a valid one, so we need something else to indicate "null-ness".
@@ -772,10 +786,15 @@ typedef enum {
      */
     CONTROL_INDEX_MIDI_PITCHBEND = 131,
 
+    /*!
+     * Special value to indicate MIDI learn.
+     */
+    CONTROL_INDEX_MIDI_LEARN = 132,
+
     /*!
      * Highest index allowed for mappings.
      */
-    CONTROL_INDEX_MAX_ALLOWED = CONTROL_INDEX_MIDI_PITCHBEND
+    CONTROL_INDEX_MAX_ALLOWED = CONTROL_INDEX_MIDI_LEARN
 
 } SpecialMappedControlIndex;
 
@@ -1141,7 +1160,25 @@ typedef enum {
      * @a value1   Parameter index
      * @a valueStr New mapped range as "%f:%f" syntax
      */
-    ENGINE_CALLBACK_PARAMETER_MAPPED_RANGE_CHANGED = 46
+    ENGINE_CALLBACK_PARAMETER_MAPPED_RANGE_CHANGED = 46,
+
+    /*!
+     * A patchbay client position has changed.
+     * @a pluginId Client Id
+     * @a value1   X position 1
+     * @a value2   Y position 1
+     * @a value3   X position 2
+     * @a valuef   Y position 2
+     */
+    ENGINE_CALLBACK_PATCHBAY_CLIENT_POSITION_CHANGED = 47,
+
+    /*!
+     * A plugin embed UI has been resized.
+     * @a pluginId Plugin Id to resize
+     * @a value1   New width
+     * @a value2   New height
+     */
+    ENGINE_CALLBACK_EMBED_UI_RESIZED = 48,
 
 } EngineCallbackOpcode;
 
@@ -1411,7 +1448,14 @@ typedef enum {
     /*!
      * Capture console output into debug callbacks.
      */
-    ENGINE_OPTION_DEBUG_CONSOLE_OUTPUT = 33
+    ENGINE_OPTION_DEBUG_CONSOLE_OUTPUT = 33,
+
+    /*!
+     * A prefix to give to all plugin clients created by Carla.
+     * Mostly useful for JACK multi-client mode.
+     * @note MUST include at least one "." (dot).
+     */
+    ENGINE_OPTION_CLIENT_NAME_PREFIX = 34
 
 } EngineOption;
 
