@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin Host
- * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 #endif
 
 #include "CarlaEngineClient.hpp"
+#include "CarlaEngineInit.hpp"
 #include "CarlaPlugin.hpp"
 
 #include "CarlaBackendUtils.hpp"
@@ -66,7 +67,10 @@ class CarlaEngineBridgeClient : public CarlaEngineClientForSubclassing
 {
 public:
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-    CarlaEngineBridgeClient(const CarlaEngine& engine, EngineInternalGraph& egraph, CarlaPlugin* const plugin, LatencyChangedCallback* const cb)
+    CarlaEngineBridgeClient(const CarlaEngine& engine,
+                            EngineInternalGraph& egraph,
+                            const CarlaPluginPtr plugin,
+                            LatencyChangedCallback* const cb)
         : CarlaEngineClientForSubclassing(engine, egraph, plugin),
           fLatencyCallback(cb) {}
 #else
@@ -293,7 +297,7 @@ public:
         fShmNonRtServerControl.commitWrite();
     }
 
-    CarlaEngineClient* addClient(CarlaPlugin* const plugin) override
+    CarlaEngineClient* addClient(const CarlaPluginPtr plugin) override
     {
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
         return new CarlaEngineBridgeClient(*this, pData->graph, plugin, this);
@@ -307,9 +311,9 @@ public:
 
     void idle() noexcept override
     {
-        CarlaPlugin* const plugin(pData->plugins[0].plugin);
+        const CarlaPluginPtr plugin = pData->plugins[0].plugin;
 
-        if (plugin == nullptr)
+        if (plugin.get() == nullptr)
         {
             if (const uint32_t length = static_cast<uint32_t>(pData->lastError.length()))
             {
@@ -701,7 +705,7 @@ public:
             break;
 
         case ENGINE_CALLBACK_RELOAD_PARAMETERS:
-            if (CarlaPlugin* const plugin = pData->plugins[0].plugin)
+            if (const CarlaPluginPtr plugin = pData->plugins[0].plugin)
             {
                 if (const uint32_t count = std::min(pData->options.maxParameters, plugin->getParameterCount()))
                 {
@@ -744,16 +748,17 @@ public:
 
     void handleNonRtData()
     {
-        CarlaPlugin* const plugin(pData->plugins[0].plugin);
-        CARLA_SAFE_ASSERT_RETURN(plugin != nullptr,);
+        const CarlaPluginPtr plugin = pData->plugins[0].plugin;
+        CARLA_SAFE_ASSERT_RETURN(plugin.get() != nullptr,);
 
         for (; fShmNonRtClientControl.isDataAvailableForReading();)
         {
-            const PluginBridgeNonRtClientOpcode opcode(fShmNonRtClientControl.readOpcode());
+            const PluginBridgeNonRtClientOpcode opcode = fShmNonRtClientControl.readOpcode();
 
 #ifdef DEBUG
             if (opcode != kPluginBridgeNonRtClientPing) {
-                carla_debug("CarlaEngineBridge::handleNonRtData() - got opcode: %s", PluginBridgeNonRtClientOpcode2str(opcode));
+                carla_debug("CarlaEngineBridge::handleNonRtData() - got opcode: %i:%s",
+                            opcode, PluginBridgeNonRtClientOpcode2str(opcode));
             }
 #endif
 
@@ -785,12 +790,12 @@ public:
             }   break;
 
             case kPluginBridgeNonRtClientActivate:
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setActive(true, false, false);
                 break;
 
             case kPluginBridgeNonRtClientDeactivate:
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setActive(false, false, false);
                 break;
 
@@ -804,7 +809,7 @@ public:
                 const uint32_t index(fShmNonRtClientControl.readUInt());
                 const float    value(fShmNonRtClientControl.readFloat());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setParameterValue(index, value, false, false, false);
                 break;
             }
@@ -813,7 +818,7 @@ public:
                 const uint32_t index(fShmNonRtClientControl.readUInt());
                 const uint8_t  channel(fShmNonRtClientControl.readByte());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setParameterMidiChannel(index, channel, false, false);
                 break;
             }
@@ -822,8 +827,8 @@ public:
                 const uint32_t index(fShmNonRtClientControl.readUInt());
                 const int16_t  ctrl(fShmNonRtClientControl.readShort());
 
-                if (plugin != nullptr && plugin->isEnabled())
-                    plugin->setParameterMappedControlIndex(index, ctrl, false, false);
+                if (plugin->isEnabled())
+                    plugin->setParameterMappedControlIndex(index, ctrl, false, false, true);
                 break;
             }
 
@@ -832,7 +837,7 @@ public:
                 const float    minimum = fShmNonRtClientControl.readFloat();
                 const float    maximum = fShmNonRtClientControl.readFloat();
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setParameterMappedRange(index, minimum, maximum, false, false);
                 break;
             }
@@ -840,7 +845,7 @@ public:
             case kPluginBridgeNonRtClientSetProgram: {
                 const int32_t index(fShmNonRtClientControl.readInt());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setProgram(index, true, false, false);
                 break;
             }
@@ -848,7 +853,7 @@ public:
             case kPluginBridgeNonRtClientSetMidiProgram: {
                 const int32_t index(fShmNonRtClientControl.readInt());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setMidiProgram(index, true, false, false);
                 break;
             }
@@ -874,7 +879,7 @@ public:
                 if (valueSize > 0)
                     fShmNonRtClientControl.readCustomData(valueStr, valueSize);
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setCustomData(typeStr, keyStr, valueStr, true);
                 break;
             }
@@ -888,7 +893,7 @@ public:
                 fShmNonRtClientControl.readCustomData(chunkFilePathTry, size);
 
                 CARLA_SAFE_ASSERT_BREAK(chunkFilePathTry[0] != '\0');
-                if (plugin == nullptr || ! plugin->isEnabled()) break;
+                if (! plugin->isEnabled()) break;
 
                 String chunkFilePath(chunkFilePathTry);
 
@@ -919,7 +924,7 @@ public:
                 const int16_t channel(fShmNonRtClientControl.readShort());
                 CARLA_SAFE_ASSERT_BREAK(channel >= -1 && channel < MAX_MIDI_CHANNELS);
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setCtrlChannel(static_cast<int8_t>(channel), false, false);
                 break;
             }
@@ -928,7 +933,7 @@ public:
                 const uint32_t option(fShmNonRtClientControl.readUInt());
                 const bool     yesNo(fShmNonRtClientControl.readBool());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->setOption(option, yesNo, false);
                 break;
             }
@@ -936,15 +941,26 @@ public:
             case kPluginBridgeNonRtClientSetOptions: {
                 const uint32_t options(fShmNonRtClientControl.readUInt());
 
-                if (plugin != nullptr)
-                    plugin->pData->options = options;
+                plugin->pData->options = options;
+                break;
+            }
+
+            case kPluginBridgeNonRtClientSetWindowTitle: {
+                const uint32_t size = fShmNonRtClientControl.readUInt();
+                CARLA_SAFE_ASSERT_BREAK(size > 0);
+
+                char title[size+1];
+                carla_zeroChars(title, size+1);
+                fShmNonRtClientControl.readCustomData(title, size);
+
+                plugin->setCustomUITitle(title);
                 break;
             }
 
             case kPluginBridgeNonRtClientGetParameterText: {
-                const int32_t index(fShmNonRtClientControl.readInt());
+                const int32_t index = fShmNonRtClientControl.readInt();
 
-                if (index >= 0 && plugin != nullptr && plugin->isEnabled())
+                if (index >= 0 && plugin->isEnabled())
                 {
                     char bufStr[STR_MAX+1];
                     carla_zeroChars(bufStr, STR_MAX+1);
@@ -964,15 +980,29 @@ public:
 
                     fShmNonRtServerControl.waitIfDataIsReachingLimit();
                 }
+                else
+                {
+                    const CarlaMutexLocker _cml(fShmNonRtServerControl.mutex);
+
+                    fShmNonRtServerControl.writeOpcode(kPluginBridgeNonRtServerSetParameterText);
+                    fShmNonRtServerControl.writeInt(index);
+                    fShmNonRtServerControl.writeUInt(0);
+                    fShmNonRtServerControl.commitWrite();
+                }
 
                 break;
             }
 
             case kPluginBridgeNonRtClientPrepareForSave: {
-                if (plugin == nullptr || ! plugin->isEnabled())
-                    break;
+                if (! plugin->isEnabled())
+                {
+                    const CarlaMutexLocker _cml(fShmNonRtServerControl.mutex);
+                    fShmNonRtServerControl.writeOpcode(kPluginBridgeNonRtServerSaved);
+                    fShmNonRtServerControl.commitWrite();
+                    return;
+                }
 
-                plugin->prepareForSave();
+                plugin->prepareForSave(false);
 
                 for (uint32_t i=0, count=plugin->getCustomDataCount(); i<count; ++i)
                 {
@@ -1045,17 +1075,17 @@ public:
             }
 
             case kPluginBridgeNonRtClientRestoreLV2State:
-                if (plugin != nullptr && plugin->isEnabled())
-                    plugin->restoreLV2State();
+                if (plugin->isEnabled())
+                    plugin->restoreLV2State(false);
                 break;
 
             case kPluginBridgeNonRtClientShowUI:
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->showCustomUI(true);
                 break;
 
             case kPluginBridgeNonRtClientHideUI:
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->showCustomUI(false);
                 break;
 
@@ -1063,7 +1093,7 @@ public:
                 const uint32_t index(fShmNonRtClientControl.readUInt());
                 const float    value(fShmNonRtClientControl.readFloat());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->uiParameterChange(index, value);
                 break;
             }
@@ -1071,7 +1101,7 @@ public:
             case kPluginBridgeNonRtClientUiProgramChange: {
                 const uint32_t index(fShmNonRtClientControl.readUInt());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->uiProgramChange(index);
                 break;
             }
@@ -1079,7 +1109,7 @@ public:
             case kPluginBridgeNonRtClientUiMidiProgramChange: {
                 const uint32_t index(fShmNonRtClientControl.readUInt());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->uiMidiProgramChange(index);
                 break;
             }
@@ -1089,7 +1119,7 @@ public:
                 const uint8_t note(fShmNonRtClientControl.readByte());
                 const uint8_t velo(fShmNonRtClientControl.readByte());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->uiNoteOn(chnl, note, velo);
                 break;
             }
@@ -1098,7 +1128,7 @@ public:
                 const uint8_t chnl(fShmNonRtClientControl.readByte());
                 const uint8_t note(fShmNonRtClientControl.readByte());
 
-                if (plugin != nullptr && plugin->isEnabled())
+                if (plugin->isEnabled())
                     plugin->uiNoteOff(chnl, note);
                 break;
             }
@@ -1134,7 +1164,7 @@ protected:
             for (; fShmRtClientControl.isDataAvailableForReading();)
             {
                 const PluginBridgeRtClientOpcode opcode(fShmRtClientControl.readOpcode());
-                CarlaPlugin* const plugin(pData->plugins[0].plugin);
+                const CarlaPluginPtr plugin = pData->plugins[0].plugin;
 
 #ifdef DEBUG
                 if (opcode != kPluginBridgeRtClientProcess && opcode != kPluginBridgeRtClientMidiEvent) {
@@ -1178,6 +1208,7 @@ protected:
                     offlineModeChanged(fIsOffline);
                     break;
 
+                // NOTE this is never used
                 case kPluginBridgeRtClientControlEventParameter: {
                     const uint32_t time(fShmRtClientControl.readUInt());
                     const uint8_t  channel(fShmRtClientControl.readByte());
@@ -1186,12 +1217,14 @@ protected:
 
                     if (EngineEvent* const event = getNextFreeInputEvent())
                     {
-                        event->type    = kEngineEventTypeControl;
-                        event->time    = time;
-                        event->channel = channel;
-                        event->ctrl.type  = kEngineControlEventTypeParameter;
-                        event->ctrl.param = param;
-                        event->ctrl.value = value;
+                        event->type                 = kEngineEventTypeControl;
+                        event->time                 = time;
+                        event->channel              = channel;
+                        event->ctrl.type            = kEngineControlEventTypeParameter;
+                        event->ctrl.param           = param;
+                        event->ctrl.midiValue       = -1;
+                        event->ctrl.normalizedValue = value;
+                        event->ctrl.handled         = true;
                     }
                     break;
                 }
@@ -1203,12 +1236,14 @@ protected:
 
                     if (EngineEvent* const event = getNextFreeInputEvent())
                     {
-                        event->type    = kEngineEventTypeControl;
-                        event->time    = time;
-                        event->channel = channel;
-                        event->ctrl.type  = kEngineControlEventTypeMidiBank;
-                        event->ctrl.param = index;
-                        event->ctrl.value = 0.0f;
+                        event->type                 = kEngineEventTypeControl;
+                        event->time                 = time;
+                        event->channel              = channel;
+                        event->ctrl.type            = kEngineControlEventTypeMidiBank;
+                        event->ctrl.param           = index;
+                        event->ctrl.midiValue       = -1;
+                        event->ctrl.normalizedValue = 0.0f;
+                        event->ctrl.handled         = true;
                     }
                     break;
                 }
@@ -1220,12 +1255,14 @@ protected:
 
                     if (EngineEvent* const event = getNextFreeInputEvent())
                     {
-                        event->type    = kEngineEventTypeControl;
-                        event->time    = time;
-                        event->channel = channel;
-                        event->ctrl.type  = kEngineControlEventTypeMidiProgram;
-                        event->ctrl.param = index;
-                        event->ctrl.value = 0.0f;
+                        event->type                 = kEngineEventTypeControl;
+                        event->time                 = time;
+                        event->channel              = channel;
+                        event->ctrl.type            = kEngineControlEventTypeMidiProgram;
+                        event->ctrl.param           = index;
+                        event->ctrl.midiValue       = -1;
+                        event->ctrl.normalizedValue = 0.0f;
+                        event->ctrl.handled         = true;
                     }
                     break;
                 }
@@ -1236,12 +1273,14 @@ protected:
 
                     if (EngineEvent* const event = getNextFreeInputEvent())
                     {
-                        event->type    = kEngineEventTypeControl;
-                        event->time    = time;
-                        event->channel = channel;
-                        event->ctrl.type  = kEngineControlEventTypeAllSoundOff;
-                        event->ctrl.param = 0;
-                        event->ctrl.value = 0.0f;
+                        event->type                 = kEngineEventTypeControl;
+                        event->time                 = time;
+                        event->channel              = channel;
+                        event->ctrl.type            = kEngineControlEventTypeAllSoundOff;
+                        event->ctrl.param           = 0;
+                        event->ctrl.midiValue       = -1;
+                        event->ctrl.normalizedValue = 0.0f;
+                        event->ctrl.handled         = true;
                     }
                 }   break;
 
@@ -1251,12 +1290,14 @@ protected:
 
                     if (EngineEvent* const event = getNextFreeInputEvent())
                     {
-                        event->type    = kEngineEventTypeControl;
-                        event->time    = time;
-                        event->channel = channel;
-                        event->ctrl.type  = kEngineControlEventTypeAllNotesOff;
-                        event->ctrl.param = 0;
-                        event->ctrl.value = 0.0f;
+                        event->type                 = kEngineEventTypeControl;
+                        event->time                 = time;
+                        event->channel              = channel;
+                        event->ctrl.type            = kEngineControlEventTypeAllNotesOff;
+                        event->ctrl.param           = 0;
+                        event->ctrl.midiValue       = -1;
+                        event->ctrl.normalizedValue = 0.0f;
+                        event->ctrl.handled         = true;
                     }
                 }   break;
 
@@ -1307,7 +1348,7 @@ protected:
 
                     CARLA_SAFE_ASSERT_BREAK(fShmAudioPool.data != nullptr);
 
-                    if (plugin != nullptr && plugin->isEnabled() && plugin->tryLock(fIsOffline))
+                    if (plugin.get() != nullptr && plugin->isEnabled() && plugin->tryLock(fIsOffline))
                     {
                         const BridgeTimeInfo& bridgeTimeInfo(fShmRtClientControl.data->timeInfo);
 
@@ -1513,11 +1554,18 @@ private:
 
 // -----------------------------------------------------------------------
 
-CarlaEngine* CarlaEngine::newBridge(const char* const audioPoolBaseName, const char* const rtClientBaseName, const char* const nonRtClientBaseName, const char* const nonRtServerBaseName)
+namespace EngineInit {
+
+CarlaEngine* newBridge(const char* const audioPoolBaseName,
+                       const char* const rtClientBaseName,
+                       const char* const nonRtClientBaseName,
+                       const char* const nonRtServerBaseName)
 {
     return new CarlaEngineBridge(audioPoolBaseName, rtClientBaseName, nonRtClientBaseName, nonRtServerBaseName);
 }
 
+}
+
 // -----------------------------------------------------------------------
 
 CARLA_BACKEND_END_NAMESPACE
