@@ -122,7 +122,7 @@ int CarlaEngineOsc::handleMessage(const bool isTCP, const char* const path, cons
     }
 
     // Get plugin
-    CarlaPlugin* const plugin(fEngine->getPluginUnchecked(pluginId));
+    const CarlaPluginPtr plugin = fEngine->getPluginUnchecked(pluginId);
 
     if (plugin == nullptr || plugin->getId() != pluginId)
     {
@@ -131,9 +131,9 @@ int CarlaEngineOsc::handleMessage(const bool isTCP, const char* const path, cons
     }
 
     // Get method from path, "/Carla/i/method" -> "method"
-    char method[32+1];
-    method[32] = '\0';
-    std::strncpy(method, path + (nameSize + offset), 32);
+    char method[48];
+    std::strncpy(method, path + (nameSize + offset), 47);
+    method[47] = '\0';
 
     if (method[0] == '\0')
     {
@@ -239,7 +239,7 @@ int CarlaEngineOsc::handleMsgRegister(const bool isTCP,
 
             for (uint i=0, count=fEngine->getCurrentPluginCount(); i < count; ++i)
             {
-                CarlaPlugin* const plugin(fEngine->getPluginUnchecked(i));
+                const CarlaPluginPtr plugin = fEngine->getPluginUnchecked(i);
                 CARLA_SAFE_ASSERT_CONTINUE(plugin != nullptr);
 
                 fEngine->callback(false, true, ENGINE_CALLBACK_PLUGIN_ADDED, i, 0, 0, 0, 0.0f, plugin->getName());
@@ -352,6 +352,25 @@ int CarlaEngineOsc::handleMsgControl(const char* const method,
 
         ok = fEngine->patchbayDisconnect(external, static_cast<uint32_t>(connectionId));
     }
+    else if (std::strcmp(method, "patchbay_set_group_pos") == 0)
+    {
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(argc == 7);
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[1] == 'i');
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[2] == 'i');
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[3] == 'i');
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[4] == 'i');
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[5] == 'i');
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(types[6] == 'i');
+
+        const bool external = argv[1]->i != 0;
+
+        const int32_t groupId = argv[2]->i;
+        CARLA_SAFE_ASSERT_RETURN_OSC_ERR(groupId >= 0);
+
+        ok = fEngine->patchbaySetGroupPos(true, false,
+                                          external, static_cast<uint32_t>(groupId),
+                                          argv[3]->i, argv[4]->i, argv[5]->i, argv[6]->i);
+    }
     else if (std::strcmp(method, "patchbay_refresh") == 0)
     {
         CARLA_SAFE_ASSERT_RETURN_OSC_ERR(argc == 2);
@@ -643,7 +662,7 @@ int CarlaEngineOsc::handleMsgSetParameterMappedControlIndex(CARLA_ENGINE_OSC_HAN
     CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
     CARLA_SAFE_ASSERT_RETURN(ctrl >= CONTROL_INDEX_NONE && ctrl <= CONTROL_INDEX_MAX_ALLOWED, 0);
 
-    plugin->setParameterMappedControlIndex(static_cast<uint32_t>(index), static_cast<int16_t>(ctrl), false, true);
+    plugin->setParameterMappedControlIndex(static_cast<uint32_t>(index), static_cast<int16_t>(ctrl), false, true, true);
     return 0;
 }
 
