@@ -21,6 +21,7 @@
 #include "CarlaBackendUtils.hpp"
 #include "CarlaBase64Utils.hpp"
 #include "CarlaMathUtils.hpp"
+#include "CarlaMIDI.h"
 #include "CarlaPluginUI.hpp"
 #include "CarlaScopeUtils.hpp"
 #include "CarlaStringList.hpp"
@@ -42,7 +43,7 @@ using water::XmlElement;
 
 CARLA_BACKEND_START_NAMESPACE
 
-// -------------------------------------------------------------------
+// -------------------------------------------------------------------------------------------------------------------
 // Fallback data
 
 static const ParameterData   kParameterDataNull   = { PARAMETER_UNKNOWN, 0x0, PARAMETER_NULL, -1, 0, CONTROL_INDEX_NONE, 0.0f, 1.0f };
@@ -53,7 +54,7 @@ static const CustomData        kCustomDataFallback        = { nullptr, nullptr,
 static /* */ CustomData        kCustomDataFallbackNC      = { nullptr, nullptr, nullptr };
 static const PluginPostRtEvent kPluginPostRtEventFallback = { kPluginPostRtEventNull, false, 0, 0, 0, 0.0f };
 
-// -------------------------------------------------------------------
+// -------------------------------------------------------------------------------------------------------------------
 // ParamSymbol struct, needed for CarlaPlugin::loadStateSave()
 
 struct ParamSymbol {
@@ -78,7 +79,7 @@ struct ParamSymbol {
 #endif
 };
 
-// -------------------------------------------------------------------
+// -------------------------------------------------------------------------------------------------------------------
 // Constructor and destructor
 
 CarlaPlugin::CarlaPlugin(CarlaEngine* const engine, const uint id)
@@ -439,7 +440,7 @@ void CarlaPlugin::getParameterCountInfo(uint32_t& ins, uint32_t& outs) const noe
 // -------------------------------------------------------------------
 // Set data (state)
 
-void CarlaPlugin::prepareForSave()
+void CarlaPlugin::prepareForSave(bool)
 {
 }
 
@@ -507,11 +508,14 @@ void CarlaPlugin::randomizeParameters() noexcept
 
 const CarlaStateSave& CarlaPlugin::getStateSave(const bool callPrepareForSave)
 {
-    if (callPrepareForSave)
-        prepareForSave();
-
     pData->stateSave.clear();
 
+    if (callPrepareForSave)
+    {
+        pData->stateSave.temporary = true;
+        prepareForSave(true);
+    }
+
     const PluginType pluginType(getType());
 
     char strBuf[STR_MAX+1];
@@ -547,11 +551,14 @@ const CarlaStateSave& CarlaPlugin::getStateSave(const bool callPrepareForSave)
     pData->stateSave.ctrlChannel  = pData->ctrlChannel;
 #endif
 
-    bool usingChunk = false;
+    if (pData->hints & PLUGIN_IS_BRIDGE)
+        waitForBridgeSaveSignal();
 
     // ---------------------------------------------------------------
     // Chunk
 
+    bool usingChunk = false;
+
     if (pData->options & PLUGIN_OPTION_USE_CHUNKS)
     {
         void* data = nullptr;
@@ -608,19 +615,22 @@ const CarlaStateSave& CarlaPlugin::getStateSave(const bool callPrepareForSave)
         stateParameter->dummy = dummy;
         stateParameter->index = paramData.index;
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-        stateParameter->mappedControlIndex = paramData.mappedControlIndex;
-        stateParameter->midiChannel        = paramData.midiChannel;
-
-        if (paramData.hints & PARAMETER_MAPPED_RANGES_SET)
+        if (paramData.mappedControlIndex != CONTROL_INDEX_MIDI_LEARN)
         {
-            stateParameter->mappedMinimum = paramData.mappedMinimum;
-            stateParameter->mappedMaximum = paramData.mappedMaximum;
-            stateParameter->mappedRangeValid = true;
+            stateParameter->mappedControlIndex = paramData.mappedControlIndex;
+            stateParameter->midiChannel        = paramData.midiChannel;
 
-            if (paramData.hints & PARAMETER_USES_SAMPLERATE)
+            if (paramData.hints & PARAMETER_MAPPED_RANGES_SET)
             {
-                stateParameter->mappedMinimum /= sampleRate;
-                stateParameter->mappedMaximum /= sampleRate;
+                stateParameter->mappedMinimum = paramData.mappedMinimum;
+                stateParameter->mappedMaximum = paramData.mappedMaximum;
+                stateParameter->mappedRangeValid = true;
+
+                if (paramData.hints & PARAMETER_USES_SAMPLERATE)
+                {
+                    stateParameter->mappedMinimum /= sampleRate;
+                    stateParameter->mappedMaximum /= sampleRate;
+                }
             }
         }
 #endif
@@ -647,9 +657,6 @@ const CarlaStateSave& CarlaPlugin::getStateSave(const bool callPrepareForSave)
     // ---------------------------------------------------------------
     // Custom Data
 
-    if (pData->hints & PLUGIN_IS_BRIDGE)
-        waitForBridgeSaveSignal();
-
     for (LinkedList<CustomData>::Itenerator it = pData->custom.begin2(); it.valid(); it.next())
     {
         const CustomData& cData(it.getValue(kCustomDataFallback));
@@ -847,7 +854,8 @@ void CarlaPlugin::loadStateSave(const CarlaStateSave& stateSave)
                                         stateParameter->mappedMaximum, true, true);
             }
 
-            setParameterMappedControlIndex(static_cast<uint32_t>(index), stateParameter->mappedControlIndex, true, true);
+            setParameterMappedControlIndex(static_cast<uint32_t>(index),
+                                           stateParameter->mappedControlIndex, true, true, false);
             setParameterMidiChannel(static_cast<uint32_t>(index), stateParameter->midiChannel, true, true);
 #endif
         }
@@ -901,7 +909,7 @@ void CarlaPlugin::loadStateSave(const CarlaStateSave& stateSave)
             if (std::strcmp(customData.type, CUSTOM_DATA_TYPE_PROPERTY) == 0)
                 continue;
 
-            restoreLV2State();
+            restoreLV2State(stateSave.temporary);
             break;
         }
     }
@@ -1721,7 +1729,9 @@ void CarlaPlugin::setParameterMidiChannel(const uint32_t parameterId, const uint
 #endif
 }
 
-void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, const int16_t index, const bool sendOsc, const bool sendCallback) noexcept
+void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, const int16_t index,
+                                                 const bool sendOsc, const bool sendCallback,
+                                                 const bool reconfigureNow) noexcept
 {
     if (pData->engineBridged) {
         CARLA_SAFE_ASSERT_RETURN(!sendOsc && !sendCallback,);
@@ -1751,6 +1761,24 @@ void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, con
     if (portNameSize < STR_MAX)
         strBuf[portNameSize] = '\0';
 
+    // was learning something else before, stop that first
+    if (pData->midiLearnParameterIndex >= 0 && pData->midiLearnParameterIndex != static_cast<int32_t>(parameterId))
+    {
+        const int32_t oldParameterId = pData->midiLearnParameterIndex;
+        pData->midiLearnParameterIndex = -1;
+
+        CARLA_SAFE_ASSERT_RETURN(oldParameterId < static_cast<int32_t>(pData->param.count),);
+
+        pData->param.data[oldParameterId].mappedControlIndex = CONTROL_INDEX_NONE;
+        pData->engine->callback(true, true,
+                                ENGINE_CALLBACK_PARAMETER_MAPPED_CONTROL_INDEX_CHANGED,
+                                pData->id,
+                                oldParameterId,
+                                CONTROL_INDEX_NONE,
+                                0, 0.0f, nullptr);
+    }
+
+    // mapping new parameter to CV
     if (index == CONTROL_INDEX_CV)
     {
         CARLA_SAFE_ASSERT_RETURN(pData->event.cvSourcePorts != nullptr,);
@@ -1760,8 +1788,9 @@ void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, con
         CarlaEngineCVPort* const cvPort =
             (CarlaEngineCVPort*)pData->client->addPort(kEnginePortTypeCV, strBuf, true, parameterId);
         cvPort->setRange(paramData.mappedMinimum, paramData.mappedMaximum);
-        pData->event.cvSourcePorts->addCVSource(cvPort, parameterId);
+        pData->event.cvSourcePorts->addCVSource(cvPort, parameterId, reconfigureNow);
     }
+    // unmapping from CV
     else if (paramData.mappedControlIndex == CONTROL_INDEX_CV)
     {
         CARLA_SAFE_ASSERT_RETURN(pData->event.cvSourcePorts != nullptr,);
@@ -1769,6 +1798,7 @@ void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, con
         CARLA_SAFE_ASSERT(pData->client->removePort(kEnginePortTypeCV, strBuf, true));
         CARLA_SAFE_ASSERT(pData->event.cvSourcePorts->removeCVSource(parameterId));
     }
+    // mapping to something new
     else if (paramData.mappedControlIndex == CONTROL_INDEX_NONE)
     {
         // when doing MIDI CC mapping, ensure ranges are within bounds
@@ -1783,16 +1813,26 @@ void CarlaPlugin::setParameterMappedControlIndex(const uint32_t parameterId, con
     paramData.mappedControlIndex = index;
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    if (index == CONTROL_INDEX_MIDI_LEARN)
+        pData->midiLearnParameterIndex = static_cast<int32_t>(parameterId);
+    else
+        pData->midiLearnParameterIndex = -1;
+
     pData->engine->callback(sendCallback, sendOsc,
                             ENGINE_CALLBACK_PARAMETER_MAPPED_CONTROL_INDEX_CHANGED,
                             pData->id,
                             static_cast<int>(parameterId),
                             index,
                             0, 0.0f, nullptr);
+#else
+    return;
+    // unused
+    (void)reconfigureNow;
 #endif
 }
 
-void CarlaPlugin::setParameterMappedRange(const uint32_t parameterId, const float minimum, const float maximum, const bool sendOsc, const bool sendCallback) noexcept
+void CarlaPlugin::setParameterMappedRange(const uint32_t parameterId, const float minimum, const float maximum,
+                                          const bool sendOsc, const bool sendCallback) noexcept
 {
     if (pData->engineBridged) {
         CARLA_SAFE_ASSERT_RETURN(!sendOsc && !sendCallback,);
@@ -2247,6 +2287,35 @@ void CarlaPlugin::idle()
                                         0, 0.0f, nullptr);
             }
         } break;
+
+        case kPluginPostRtEventMidiLearn: {
+            CARLA_SAFE_ASSERT_BREAK(event.value1 >= 0 && event.value1 < MAX_MIDI_CHANNELS);
+            CARLA_SAFE_ASSERT_BREAK(event.value2 >= 0 && event.value2 < MAX_MIDI_NOTE);
+
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+            if (event.sendCallback)
+            {
+                const int32_t parameterId = event.value1;
+                const int32_t midiCC      = event.value2;
+                const int32_t midiChannel = event.value3;
+
+                pData->engine->callback(true, true,
+                                        ENGINE_CALLBACK_PARAMETER_MAPPED_CONTROL_INDEX_CHANGED,
+                                        pData->id,
+                                        parameterId,
+                                        midiCC,
+                                        0, 0.0f, nullptr);
+
+                pData->engine->callback(true, true,
+                                        ENGINE_CALLBACK_PARAMETER_MIDI_CHANNEL_CHANGED,
+                                        pData->id,
+                                        parameterId,
+                                        midiChannel,
+                                        0, 0.0f, nullptr);
+
+            }
+#endif
+        } break;
         }
     }
 }
@@ -2355,6 +2424,11 @@ void CarlaPlugin::postponeRtAllNotesOff()
 // -------------------------------------------------------------------
 // UI Stuff
 
+void CarlaPlugin::setCustomUITitle(const char* const title) noexcept
+{
+    pData->uiTitle = title;
+}
+
 void CarlaPlugin::showCustomUI(const bool yesNo)
 {
     if (yesNo) {
@@ -2362,6 +2436,11 @@ void CarlaPlugin::showCustomUI(const bool yesNo)
     }
 }
 
+void* CarlaPlugin::embedCustomUI(void*)
+{
+    return nullptr;
+}
+
 void CarlaPlugin::uiIdle()
 {
     if (pData->hints & PLUGIN_NEEDS_UI_MAIN_THREAD)
@@ -2384,6 +2463,7 @@ void CarlaPlugin::uiIdle()
             {
             case kPluginPostRtEventNull:
             case kPluginPostRtEventDebug:
+            case kPluginPostRtEventMidiLearn:
                 break;
 
             case kPluginPostRtEventParameterChange:
@@ -2424,8 +2504,17 @@ void CarlaPlugin::uiIdle()
 
     carla_stdout("Trying to get window...");
 
-    CarlaString uiTitle(pData->name);
-    uiTitle += " (GUI)";
+    CarlaString uiTitle;
+
+    if (pData->uiTitle.isNotEmpty())
+    {
+        uiTitle = pData->uiTitle;
+    }
+    else
+    {
+        uiTitle  = pData->name;
+        uiTitle += " (GUI)";
+    }
 
     if (CarlaPluginUI::tryTransientWinIdMatch(getUiBridgeProcessId(), uiTitle,
                                               pData->engine->getOptions().frontendWinId, pData->transientFirstTry))
@@ -2508,6 +2597,33 @@ CarlaEngineEventPort* CarlaPlugin::getDefaultEventOutPort() const noexcept
     return pData->event.portOut;
 }
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+void CarlaPlugin::checkForMidiLearn(EngineEvent& event) noexcept
+{
+    if (pData->midiLearnParameterIndex < 0)
+        return;
+    if (event.ctrl.param == MIDI_CONTROL_BANK_SELECT || event.ctrl.param == MIDI_CONTROL_BANK_SELECT__LSB)
+        return;
+    if (event.ctrl.param >= MAX_MIDI_CONTROL)
+        return;
+
+    const uint32_t parameterId = static_cast<uint32_t>(pData->midiLearnParameterIndex);
+    CARLA_SAFE_ASSERT_UINT2_RETURN(parameterId < pData->param.count, parameterId, pData->param.count,);
+
+    ParameterData& paramData(pData->param.data[parameterId]);
+    CARLA_SAFE_ASSERT_INT_RETURN(paramData.mappedControlIndex == CONTROL_INDEX_MIDI_LEARN,
+                                 paramData.mappedControlIndex,);
+
+    event.ctrl.handled = true;
+    paramData.mappedControlIndex = static_cast<int16_t>(event.ctrl.param);
+    paramData.midiChannel = event.channel;
+
+    pData->postponeRtEvent(kPluginPostRtEventMidiLearn, true,
+                           pData->midiLearnParameterIndex, event.ctrl.param, event.channel, 0.0f);
+    pData->midiLearnParameterIndex = -1;
+}
+#endif
+
 void* CarlaPlugin::getNativeHandle() const noexcept
 {
     return nullptr;
@@ -2537,9 +2653,23 @@ void CarlaPlugin::setPatchbayNodeId(const uint32_t nodeId) noexcept
 
 // -------------------------------------------------------------------
 
-void CarlaPlugin::restoreLV2State() noexcept
+void CarlaPlugin::cloneLV2Files(const CarlaPlugin&)
 {
-    carla_stderr2("Warning: restoreLV2State() called for non-implemented type");
+    carla_stderr2("Warning: cloneLV2Files() called for non-implemented type");
+}
+
+void CarlaPlugin::restoreLV2State(const bool temporary) noexcept
+{
+    carla_stderr2("Warning: restoreLV2State(%s) called for non-implemented type", bool2str(temporary));
+}
+
+void CarlaPlugin::prepareForDeletion() noexcept
+{
+    carla_debug("CarlaPlugin::prepareForDeletion");
+
+    const CarlaMutexLocker cml(pData->masterMutex);
+
+    pData->client->deactivate(true);
 }
 
 void CarlaPlugin::waitForBridgeSaveSignal() noexcept
@@ -2566,7 +2696,7 @@ CarlaPlugin::ScopedDisabler::ScopedDisabler(CarlaPlugin* const plugin) noexcept
         plugin->pData->enabled = false;
 
         if (plugin->pData->client->isActive())
-            plugin->pData->client->deactivate();
+            plugin->pData->client->deactivate(false);
     }
 }
 
