@@ -86,7 +86,8 @@ enum PluginPostRtEventType {
     kPluginPostRtEventProgramChange,     // index
     kPluginPostRtEventMidiProgramChange, // index
     kPluginPostRtEventNoteOn,            // channel, note, velo
-    kPluginPostRtEventNoteOff            // channel, note
+    kPluginPostRtEventNoteOff,           // channel, note
+    kPluginPostRtEventMidiLearn          // param, cc, channel
 };
 
 /*!
@@ -181,7 +182,8 @@ struct PluginParameterData {
     void createNew(uint32_t newCount, bool withSpecial);
     void clear() noexcept;
     float getFixedValue(uint32_t parameterId, float value) const noexcept;
-    float getFinalUnnormalizedValue(uint32_t parameterId, float value) const noexcept;
+    float getFinalUnnormalizedValue(uint32_t parameterId, float normalizedValue) const noexcept;
+    float getFinalValueWithMidiDelta(uint32_t parameterId, float value, int8_t delta) const noexcept;
 
     CARLA_DECLARE_NON_COPY_STRUCT(PluginParameterData)
 };
@@ -244,8 +246,9 @@ struct CarlaPlugin::ProtectedData {
     int8_t ctrlChannel;
     uint   extraHints;
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-    uint   transientTryCounter;
-    bool   transientFirstTry;
+    int32_t midiLearnParameterIndex;
+    uint    transientTryCounter;
+    bool    transientFirstTry;
 #endif
 
     // data 1
@@ -269,6 +272,8 @@ struct CarlaPlugin::ProtectedData {
 
     CarlaStateSave stateSave;
 
+    CarlaString uiTitle;
+
     struct ExternalNotes {
         CarlaMutex mutex;
         RtLinkedList<ExternalMidiNote>::Pool dataPool;
