@@ -18,28 +18,26 @@
 #include "CarlaPluginInternal.hpp"
 #include "CarlaEngine.hpp"
 
-#if defined(USING_JUCE)
+#ifdef USING_JUCE
 
 #include "CarlaBackendUtils.hpp"
 #include "CarlaMathUtils.hpp"
+#include "CarlaProcessUtils.hpp"
 #include "CarlaScopeUtils.hpp"
 
-#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#if defined(__clang__)
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wfloat-equal"
+# pragma clang diagnostic ignored "-Wimplicit-int-float-conversion"
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
 # pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wcast-qual"
 # pragma GCC diagnostic ignored "-Wconversion"
 # pragma GCC diagnostic ignored "-Wdouble-promotion"
 # pragma GCC diagnostic ignored "-Weffc++"
 # pragma GCC diagnostic ignored "-Wfloat-equal"
-# pragma GCC diagnostic ignored "-Woverloaded-virtual"
-# pragma GCC diagnostic ignored "-Wsign-conversion"
-# pragma GCC diagnostic ignored "-Wundef"
-# pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
-# if __GNUC__ > 7
-#  pragma GCC diagnostic ignored "-Wclass-memaccess"
-# endif
 #endif
 
+#define JUCE_GUI_BASICS_INCLUDE_XHEADERS 1
 #include "AppConfig.h"
 #include "juce_audio_processors/juce_audio_processors.h"
 #include "juce_gui_basics/juce_gui_basics.h"
@@ -50,10 +48,6 @@
 
 #include "JucePluginWindow.hpp"
 
-namespace juce {
-extern bool juce_isRunningInWine();
-}
-
 CARLA_BACKEND_START_NAMESPACE
 
 // -------------------------------------------------------------------------------------------------------------------
@@ -61,6 +55,46 @@ CARLA_BACKEND_START_NAMESPACE
 
 static const ExternalMidiNote kExternalMidiNoteFallback = { -1, 0, 0 };
 
+// -------------------------------------------------------------------------------------------------------------------
+// find all available plugin audio ports
+
+static void findMaxTotalChannels(juce::AudioProcessor* const filter,
+                                 const bool isAU, uint32_t& maxTotalIns, uint32_t& maxTotalOuts)
+{
+    filter->enableAllBuses();
+
+    if (isAU)
+    {
+        maxTotalIns  = static_cast<uint32_t>(juce::jmax(0, filter->getTotalNumInputChannels()));
+        maxTotalOuts = static_cast<uint32_t>(juce::jmax(0, filter->getTotalNumOutputChannels()));
+        return;
+    }
+
+    const int numInputBuses  = filter->getBusCount(true);
+    const int numOutputBuses = filter->getBusCount(false);
+
+    if (numInputBuses > 1 || numOutputBuses > 1)
+    {
+        maxTotalIns = maxTotalOuts = 0;
+
+        for (int i = 0; i < numInputBuses; ++i)
+            maxTotalIns += static_cast<uint32_t>(juce::jmax(0, filter->getChannelCountOfBus(true, i)));
+
+        for (int i = 0; i < numOutputBuses; ++i)
+            maxTotalOuts += static_cast<uint32_t>(juce::jmax(0, filter->getChannelCountOfBus(false, i)));
+    }
+    else
+    {
+        maxTotalIns  = numInputBuses  > 0
+                     ? static_cast<uint32_t>(juce::jmax(0, filter->getBus(true, 0)->getMaxSupportedChannels(64)))
+                     : 0;
+
+        maxTotalOuts = numOutputBuses > 0
+                     ? static_cast<uint32_t>(juce::jmax(0, filter->getBus(false, 0)->getMaxSupportedChannels(64)))
+                     : 0;
+    }
+}
+
 // -------------------------------------------------------------------------------------------------------------------
 
 class CarlaPluginJuce : public CarlaPlugin,
@@ -71,8 +105,8 @@ public:
     CarlaPluginJuce(CarlaEngine* const engine, const uint id)
         : CarlaPlugin(engine, id),
           fDesc(),
-          fInstance(nullptr),
           fFormatManager(),
+          fInstance(),
           fAudioBuffer(),
           fMidiBuffer(),
           fPosInfo(),
@@ -99,7 +133,7 @@ public:
         pData->masterMutex.lock();
 
         if (pData->client != nullptr && pData->client->isActive())
-            pData->client->deactivate();
+            pData->client->deactivate(true);
 
         if (pData->active)
         {
@@ -107,12 +141,7 @@ public:
             pData->active = false;
         }
 
-        if (fInstance != nullptr)
-        {
-            delete fInstance;
-            fInstance = nullptr;
-        }
-
+        fInstance = nullptr;
         clearBuffers();
     }
 
@@ -192,6 +221,7 @@ public:
             options |= PLUGIN_OPTION_SEND_PITCHBEND;
             options |= PLUGIN_OPTION_SEND_ALL_SOUND_OFF;
             options |= PLUGIN_OPTION_SEND_PROGRAM_CHANGES;
+            options |= PLUGIN_OPTION_SKIP_SENDING_NOTES;
         }
 
         return options;
@@ -271,12 +301,12 @@ public:
     {
         CarlaPlugin::setName(newName);
 
-        if (fWindow != nullptr)
-        {
-            juce::String uiName(pData->name);
-            uiName += " (GUI)";
-            fWindow->setName(uiName);
-        }
+        if (fWindow == nullptr || pData->uiTitle.isNotEmpty())
+            return;
+
+        juce::String uiName(pData->name);
+        uiName += " (GUI)";
+        fWindow->setName(uiName);
     }
 
     // -------------------------------------------------------------------
@@ -381,29 +411,55 @@ public:
     // -------------------------------------------------------------------
     // Set ui stuff
 
+    void setCustomUITitle(const char* const title) noexcept override
+    {
+        if (fWindow != nullptr)
+        {
+            try {
+                fWindow->setName(title);
+            } CARLA_SAFE_EXCEPTION("set custom ui title");
+        }
+
+        CarlaPlugin::setCustomUITitle(title);
+    }
+
     void showCustomUI(const bool yesNo) override
     {
         CARLA_SAFE_ASSERT_RETURN(fInstance != nullptr,);
+        carla_debug("CarlaPluginJuce::showCustomUI(%s)", bool2str(yesNo));
 
         if (yesNo)
         {
-            if (fWindow == nullptr)
-            {
-                juce::String uiName(pData->name);
-                uiName += " (GUI)";
-
-                fWindow = new JucePluginWindow(pData->engine->getOptions().frontendWinId);
-                fWindow->setName(uiName);
-            }
-
             if (juce::AudioProcessorEditor* const editor = fInstance->createEditorIfNeeded())
+            {
+                const EngineOptions& opts(pData->engine->getOptions());
+
+                editor->setScaleFactor(opts.uiScale);
+
+                if (fWindow == nullptr)
+                {
+                    juce::String uiName;
+
+                    if (pData->uiTitle.isNotEmpty())
+                    {
+                        uiName = pData->uiTitle.buffer();
+                    }
+                    else
+                    {
+                        uiName  = pData->name;
+                        uiName += " (GUI)";
+                    }
+
+                    fWindow = new JucePluginWindow(opts.frontendWinId);
+                    fWindow->setName(uiName);
+                }
+
                 fWindow->show(editor);
+                fWindow->toFront(true);
+            }
         }
         else
         {
-            if (fWindow != nullptr)
-                fWindow->hide();
-
             if (juce::AudioProcessorEditor* const editor = fInstance->getActiveEditor())
                 delete editor;
 
@@ -448,17 +504,16 @@ public:
 
         clearBuffers();
 
-        fInstance->enableAllBuses();
-        fInstance->refreshParameterList();
-
         uint32_t aIns, aOuts, mIns, mOuts, params;
         mIns = mOuts = 0;
 
         bool needsCtrlIn, needsCtrlOut;
         needsCtrlIn = needsCtrlOut = false;
 
-        aIns   = static_cast<uint32_t>(std::max(fInstance->getTotalNumInputChannels(), 0));
-        aOuts  = static_cast<uint32_t>(std::max(fInstance->getTotalNumOutputChannels(), 0));
+        const bool isAU = fDesc.pluginFormatName == "AU" || fDesc.pluginFormatName == "AudioUnit";
+        findMaxTotalChannels(fInstance.get(), isAU, aIns, aOuts);
+        fInstance->refreshParameterList();
+
         params = static_cast<uint32_t>(std::max(fInstance->getNumParameters(), 0));
 
         if (fInstance->acceptsMidi())
@@ -656,7 +711,10 @@ public:
         if (mOuts > 0)
             pData->extraHints |= PLUGIN_EXTRA_HINT_HAS_MIDI_OUT;
 
-        fInstance->setPlayConfigDetails(static_cast<int>(aIns), static_cast<int>(aOuts), pData->engine->getSampleRate(), static_cast<int>(pData->engine->getBufferSize()));
+        fInstance->setPlayConfigDetails(static_cast<int>(aIns),
+                                        static_cast<int>(aOuts),
+                                        pData->engine->getSampleRate(),
+                                        static_cast<int>(pData->engine->getBufferSize()));
 
         bufferSizeChanged(pData->engine->getBufferSize());
         reloadPrograms(true);
@@ -834,7 +892,7 @@ public:
 
             for (uint32_t i=0, numEvents=pData->event.portIn->getEventCount(); i < numEvents; ++i)
             {
-                const EngineEvent& event(pData->event.portIn->getEvent(i));
+                EngineEvent& event(pData->event.portIn->getEvent(i));
 
                 if (event.time >= frames)
                     continue;
@@ -845,7 +903,7 @@ public:
                     break;
 
                 case kEngineEventTypeControl: {
-                    const EngineControlEvent& ctrlEvent(event.ctrl);
+                    EngineControlEvent& ctrlEvent(event.ctrl);
 
                     switch (ctrlEvent.type)
                     {
@@ -862,7 +920,8 @@ public:
                             const uint32_t k = ctrlEvent.param;
                             CARLA_SAFE_ASSERT_CONTINUE(k < pData->param.count);
 
-                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.value);
+                            ctrlEvent.handled = true;
+                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.normalizedValue);
                             setParameterValueRT(k, value, true);
                             continue;
                         }
@@ -872,20 +931,20 @@ public:
                         {
                             if (MIDI_IS_CONTROL_BREATH_CONTROLLER(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_DRYWET) != 0)
                             {
-                                value = ctrlEvent.value;
+                                ctrlEvent.handled = true;
+                                value = ctrlEvent.normalizedValue;
                                 setDryWetRT(value, true);
                             }
-
-                            if (MIDI_IS_CONTROL_CHANNEL_VOLUME(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_VOLUME) != 0)
+                            else if (MIDI_IS_CONTROL_CHANNEL_VOLUME(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_VOLUME) != 0)
                             {
-                                value = ctrlEvent.value*127.0f/100.0f;
+                                ctrlEvent.handled = true;
+                                value = ctrlEvent.normalizedValue*127.0f/100.0f;
                                 setVolumeRT(value, true);
                             }
-
-                            if (MIDI_IS_CONTROL_BALANCE(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_BALANCE) != 0)
+                            else if (MIDI_IS_CONTROL_BALANCE(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_BALANCE) != 0)
                             {
                                 float left, right;
-                                value = ctrlEvent.value/0.5f - 1.0f;
+                                value = ctrlEvent.normalizedValue/0.5f - 1.0f;
 
                                 if (value < 0.0f)
                                 {
@@ -903,6 +962,7 @@ public:
                                     right = 1.0f;
                                 }
 
+                                ctrlEvent.handled = true;
                                 setBalanceLeftRT(left, true);
                                 setBalanceRightRT(right, true);
                             }
@@ -921,7 +981,8 @@ public:
                             if ((pData->param.data[k].hints & PARAMETER_IS_AUTOMABLE) == 0)
                                 continue;
 
-                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.value);
+                            ctrlEvent.handled = true;
+                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.normalizedValue);
                             setParameterValueRT(k, value, true);
                         }
 
@@ -930,11 +991,15 @@ public:
                             uint8_t midiData[3];
                             midiData[0] = uint8_t(MIDI_STATUS_CONTROL_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
                             midiData[1] = uint8_t(ctrlEvent.param);
-                            midiData[2] = uint8_t(ctrlEvent.value*127.0f);
+                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f);
 
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
                         }
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+                        if (! ctrlEvent.handled)
+                            checkForMidiLearn(event);
+#endif
                         break;
                     } // case kEngineControlEventTypeParameter
 
@@ -948,7 +1013,7 @@ public:
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
 
                             midiData[1] = MIDI_CONTROL_BANK_SELECT__LSB;
-                            midiData[2] = uint8_t(ctrlEvent.value*127.0f);
+                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f);
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
                         }
                         break;
@@ -965,7 +1030,7 @@ public:
                         {
                             uint8_t midiData[3];
                             midiData[0] = uint8_t(MIDI_STATUS_PROGRAM_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
-                            midiData[1] = uint8_t(ctrlEvent.value*127.0f);
+                            midiData[1] = uint8_t(ctrlEvent.normalizedValue*127.0f);
                             fMidiBuffer.addEvent(midiData, 2, static_cast<int>(event.time));
                         }
                         break;
@@ -1012,6 +1077,8 @@ public:
 
                     uint8_t status = uint8_t(MIDI_GET_STATUS_FROM_DATA(midiData));
 
+                    if ((status == MIDI_STATUS_NOTE_OFF || status == MIDI_STATUS_NOTE_ON) && (pData->options & PLUGIN_OPTION_SKIP_SENDING_NOTES))
+                        continue;
                     if (status == MIDI_STATUS_CHANNEL_PRESSURE && (pData->options & PLUGIN_OPTION_SEND_CHANNEL_PRESSURE) == 0)
                         continue;
                     if (status == MIDI_STATUS_CONTROL_CHANGE && (pData->options & PLUGIN_OPTION_SEND_CONTROL_CHANGES) == 0)
@@ -1257,7 +1324,9 @@ protected:
     // -------------------------------------------------------------------
 
 public:
-    bool init(const char* const filename, const char* const name, const char* const label, const int64_t uniqueId, const uint options, const char* const format)
+    bool init(const CarlaPluginPtr plugin,
+              const char* const filename, const char* const name, const char* const label,
+              const int64_t uniqueId, const uint options, const char* const format)
     {
         CARLA_SAFE_ASSERT_RETURN(pData->engine != nullptr, false);
 
@@ -1305,7 +1374,7 @@ public:
 
 #ifdef CARLA_OS_WIN
             // Fix for wine usage
-            if (juce::juce_isRunningInWine() && filename[0] == '/')
+            if (juce::File("Z:\\usr\\").isDirectory() && filename[0] == '/')
             {
                 jfilename.replace("/", "\\");
                 jfilename = "Z:" + jfilename;
@@ -1323,8 +1392,23 @@ public:
         {
             juce::OwnedArray<juce::PluginDescription> pluginDescriptions;
             juce::KnownPluginList plist;
-            for (int i = 0; i < fFormatManager.getNumFormats(); ++i)
-                plist.scanAndAddFile(fileOrIdentifier, true, pluginDescriptions, *fFormatManager.getFormat(i));
+
+            {
+                const ScopedAbortCatcher sac;
+
+                for (int i = 0; i < fFormatManager.getNumFormats(); ++i)
+                {
+                    try {
+                        plist.scanAndAddFile(fileOrIdentifier, true, pluginDescriptions, *fFormatManager.getFormat(i));
+                    } CARLA_SAFE_EXCEPTION_CONTINUE("scanAndAddFile")
+
+                    if (sac.wasTriggered())
+                    {
+                        pluginDescriptions.clearQuick(false);
+                        break;
+                    }
+                }
+            }
 
             if (pluginDescriptions.size() == 0)
             {
@@ -1339,10 +1423,20 @@ public:
             fDesc.uid = static_cast<int>(uniqueId);
 
         juce::String error;
-        fInstance = fFormatManager.createPluginInstance(fDesc,
-                                                        pData->engine->getSampleRate(),
-                                                        static_cast<int>(pData->engine->getBufferSize()),
-                                                        error);
+
+        {
+            const ScopedAbortCatcher sac;
+
+            try {
+                fInstance = fFormatManager.createPluginInstance(fDesc,
+                                                                pData->engine->getSampleRate(),
+                                                                static_cast<int>(pData->engine->getBufferSize()),
+                                                                error);
+            } CARLA_SAFE_EXCEPTION("createPluginInstance")
+
+            if (sac.wasTriggered())
+                fInstance = nullptr;
+        }
 
         if (fInstance == nullptr)
         {
@@ -1370,7 +1464,7 @@ public:
         // ---------------------------------------------------------------
         // register client
 
-        pData->client = pData->engine->addClient(this);
+        pData->client = pData->engine->addClient(plugin);
 
         if (pData->client == nullptr || ! pData->client->isOk())
         {
@@ -1399,6 +1493,8 @@ public:
                 pData->options |= PLUGIN_OPTION_SEND_ALL_SOUND_OFF;
             if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_PROGRAM_CHANGES))
                 pData->options |= PLUGIN_OPTION_SEND_PROGRAM_CHANGES;
+            if (isPluginOptionInverseEnabled(options, PLUGIN_OPTION_SKIP_SENDING_NOTES))
+                pData->options |= PLUGIN_OPTION_SKIP_SENDING_NOTES;
         }
 
         if (fInstance->getNumPrograms() > 1 && ((pData->options & PLUGIN_OPTION_SEND_PROGRAM_CHANGES) == 0))
@@ -1412,8 +1508,8 @@ public:
 
 private:
     juce::PluginDescription    fDesc;
-    juce::AudioPluginInstance* fInstance;
     juce::AudioPluginFormatManager fFormatManager;
+    std::unique_ptr<juce::AudioPluginInstance> fInstance;
 
     juce::AudioSampleBuffer fAudioBuffer;
     juce::MidiBuffer        fMidiBuffer;
@@ -1466,18 +1562,16 @@ CARLA_BACKEND_END_NAMESPACE
 
 CARLA_BACKEND_START_NAMESPACE
 
-CarlaPlugin* CarlaPlugin::newJuce(const Initializer& init, const char* const format)
+CarlaPluginPtr CarlaPlugin::newJuce(const Initializer& init, const char* const format)
 {
-    carla_debug("CarlaPlugin::newJuce({%p, \"%s\", \"%s\", \"%s\", " P_INT64 "}, %s)", init.engine, init.filename, init.name, init.label, init.uniqueId, format);
+    carla_debug("CarlaPlugin::newJuce({%p, \"%s\", \"%s\", \"%s\", " P_INT64 "}, %s)",
+                init.engine, init.filename, init.name, init.label, init.uniqueId, format);
 
 #ifdef USING_JUCE
-    CarlaPluginJuce* const plugin(new CarlaPluginJuce(init.engine, init.id));
+    std::shared_ptr<CarlaPluginJuce> plugin(new CarlaPluginJuce(init.engine, init.id));
 
-    if (! plugin->init(init.filename, init.name, init.label, init.uniqueId, init.options, format))
-    {
-        delete plugin;
+    if (! plugin->init(plugin, init.filename, init.name, init.label, init.uniqueId, init.options, format))
         return nullptr;
-    }
 
     return plugin;
 #else
