@@ -65,13 +65,16 @@ const char* carla_get_complete_license_text()
         "<li>LADSPA plugin support</li>"
         "<li>DSSI plugin support</li>"
         "<li>LV2 plugin support</li>"
-#if defined(USING_JUCE) && (defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN))
-        "<li>VST2/3 plugin support (using Juce)</li>"
+#if defined(USING_JUCE) && JUCE_PLUGINHOST_VST
+        "<li>VST2 plugin support (using JUCE)</li>"
 #else
         "<li>VST2 plugin support (using VeSTige header by Javier Serrano Polo)</li>"
 #endif
-#if defined(USING_JUCE) && defined(CARLA_OS_MAC)
-        "<li>AU plugin support (using Juce)</li>"
+#if defined(USING_JUCE) && JUCE_PLUGINHOST_VST3
+        "<li>VST3 plugin support (using JUCE)</li>"
+#endif
+#if defined(USING_JUCE) && JUCE_PLUGINHOST_AU
+        "<li>AU plugin support (using JUCE)</li>"
 #endif
 
         // Sample kit libraries
@@ -140,6 +143,9 @@ const char* const* carla_get_supported_file_extensions()
 #ifdef HAVE_FLUIDSYNTH
         "sf2", "sf3",
 #endif
+#ifdef HAVE_FLUIDSYNTH_INSTPATCH
+        "dls", "gig",
+#endif
 #ifdef HAVE_ZYN_DEPS
         "xmz", "xiz",
 #endif
@@ -185,6 +191,9 @@ const char* const* carla_get_supported_features()
 #ifdef HAVE_FLUIDSYNTH
         "sf2",
 #endif
+#ifdef HAVE_FLUIDSYNTH_INSTPATCH
+        "dls", "gig",
+#endif
 #ifdef HAVE_HYLIA
         "link",
 #endif
@@ -199,9 +208,7 @@ const char* const* carla_get_supported_features()
 #endif
 #ifdef USING_JUCE
         "juce",
-# if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
         "vst3",
-# endif
 # if defined(CARLA_OS_MAC)
         "au",
 # endif
