@@ -16,13 +16,6 @@
 #
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
-# ------------------------------------------------------------------------------------------------------------
-# Config
-
-# These will be modified during install
-X_LIBDIR_X = None
-X_DATADIR_X = None
-
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Global)
 
@@ -31,13 +24,8 @@ import sys
 
 from math import fmod
 
-from PyQt5.Qt import PYQT_VERSION_STR
-from PyQt5.QtCore import qFatal, QT_VERSION, QT_VERSION_STR, qWarning, QDir, QSettings
-from PyQt5.QtGui import QIcon
-from PyQt5.QtWidgets import QFileDialog, QMessageBox
-
 # ------------------------------------------------------------------------------------------------------------
-# Import Signal
+# Imports (Signal)
 
 from signal import signal, SIGINT, SIGTERM
 
@@ -47,10 +35,32 @@ try:
 except:
     haveSIGUSR1 = False
 
+# ------------------------------------------------------------------------------------------------------------
+# Imports (PyQt5)
+
+from PyQt5.Qt import PYQT_VERSION_STR
+from PyQt5.QtCore import qFatal, QT_VERSION, QT_VERSION_STR, qWarning, QDir, QSettings
+from PyQt5.QtGui import QIcon
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
+
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Custom)
 
-from carla_backend_qt import *
+from carla_backend import (
+    kIs64bit, HAIKU, LINUX, MACOS, WINDOWS,
+    MAX_DEFAULT_PARAMETERS,
+    ENGINE_PROCESS_MODE_MULTIPLE_CLIENTS,
+    ENGINE_PROCESS_MODE_PATCHBAY,
+    ENGINE_TRANSPORT_MODE_INTERNAL,
+    ENGINE_TRANSPORT_MODE_JACK
+)
+
+# ------------------------------------------------------------------------------------------------------------
+# Config
+
+# These will be modified during install
+X_LIBDIR_X = None
+X_DATADIR_X = None
 
 # ------------------------------------------------------------------------------------------------------------
 # Platform specific stuff
@@ -61,7 +71,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.1.1"
+VERSION = "2.2.0"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -126,7 +136,7 @@ MIDI_CC_LIST = (
     "04 [0x04] Foot",
     "05 [0x05] Portamento",
     "07 [0x07] Volume",
-    "02 [0x08] Balance",
+    "08 [0x08] Balance",
     "10 [0x0A] Pan",
     "11 [0x0B] Expression",
     "12 [0x0C] FX Control 1",
@@ -298,7 +308,7 @@ else:
     CARLA_DEFAULT_TRANSPORT_MODE = ENGINE_TRANSPORT_MODE_INTERNAL
 
 # OSC
-CARLA_DEFAULT_OSC_ENABLED = not WINDOWS
+CARLA_DEFAULT_OSC_ENABLED = not (MACOS or WINDOWS)
 CARLA_DEFAULT_OSC_TCP_PORT_ENABLED = True
 CARLA_DEFAULT_OSC_TCP_PORT_NUMBER  = 22752
 CARLA_DEFAULT_OSC_TCP_PORT_RANDOM  = False
@@ -487,19 +497,21 @@ readEnvVars = True
 
 if WINDOWS:
     # Check if running Wine. If yes, ignore env vars
+    # pylint: disable=import-error
     from winreg import ConnectRegistry, OpenKey, CloseKey, HKEY_CURRENT_USER
-    reg = ConnectRegistry(None, HKEY_CURRENT_USER)
+    # pylint: enable=import-error
+    _reg = ConnectRegistry(None, HKEY_CURRENT_USER)
 
     try:
-        key = OpenKey(reg, r"SOFTWARE\Wine")
-        CloseKey(key)
-        del key
+        _key = OpenKey(_reg, r"SOFTWARE\Wine")
+        CloseKey(_key)
+        del _key
         readEnvVars = False
     except:
         pass
 
-    CloseKey(reg)
-    del reg
+    CloseKey(_reg)
+    del _reg
 
 if readEnvVars:
     CARLA_DEFAULT_LADSPA_PATH = os.getenv("LADSPA_PATH", DEFAULT_LADSPA_PATH).split(splitter)
@@ -533,19 +545,15 @@ del DEFAULT_SFZ_PATH
 # ------------------------------------------------------------------------------------------------------------
 # Global Carla object
 
-class CarlaObject(object):
-    __slots__ = [
-        'gui',   # Host Window
-        'nogui', # Skip UI
-        'term',  # Terminated by OS signal
-        'utils'  # Utils object
-    ]
+class CarlaObject():
+    def __init__(self):
+        self.cnprefix = ""    # Client name prefix
+        self.gui      = None  # Host Window
+        self.nogui    = False # Skip UI
+        self.term     = False # Terminated by OS signal
+        self.utils    = None  # Utils object
 
 gCarla = CarlaObject()
-gCarla.gui   = None
-gCarla.nogui = False
-gCarla.term  = False
-gCarla.utils = None
 
 # ------------------------------------------------------------------------------------------------------------
 # Set CWD
@@ -583,7 +591,7 @@ def countDecimalPoints(step, stepSmall):
 
     count = 0
     value = fmod(abs(stepSmall), 1)
-    while value > 0.001 and value < 0.99 and count < 6:
+    while 0.001 < value < 0.99 and count < 6:
         value = fmod(value*10, 1)
         count += 1
 
@@ -605,10 +613,9 @@ def isNumber(value):
 def toList(value):
     if value is None:
         return []
-    elif not isinstance(value, list):
+    if not isinstance(value, list):
         return [value]
-    else:
-        return value
+    return value
 
 # ------------------------------------------------------------------------------------------------------------
 # Get Icon from user theme, using our own as backup (Oxygen)
@@ -622,6 +629,7 @@ def getIcon(icon, size = 16):
 def handleInitialCommandLineArguments(file):
     initName  = os.path.basename(file) if (file is not None and os.path.dirname(file) in PATH) else sys.argv[0]
     libPrefix = None
+    readPrefixNext = False
 
     for arg in sys.argv[1:]:
         if arg.startswith("--with-appname="):
@@ -630,12 +638,24 @@ def handleInitialCommandLineArguments(file):
         elif arg.startswith("--with-libprefix="):
             libPrefix = arg.replace("--with-libprefix=", "")
 
+        elif arg.startswith("--osc-gui="):
+            gCarla.nogui = int(arg.replace("--osc-gui=", ""))
+
+        elif arg.startswith("--cnprefix="):
+            gCarla.cnprefix = arg.replace("--cnprefix=", "")
+
+        elif arg == "--cnprefix":
+            readPrefixNext = True
+
         elif arg == "--gdb":
             pass
 
         elif arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui"):
             gCarla.nogui = True
 
+        elif MACOS and arg.startswith("-psn_"):
+            pass
+
         elif arg in ("-h", "--h", "-help", "--help"):
             print("Usage: %s [OPTION]... [FILE|URL]" % initName)
             print("")
@@ -643,11 +663,12 @@ def handleInitialCommandLineArguments(file):
             print("")
             print(" and OPTION can be one or more of the following:")
             print("")
-            print("    --gdb    \t Run Carla inside gdb.")
-            print(" -n,--no-gui \t Run Carla headless, don't show UI.")
+            print("    --cnprefix\t Set a prefix for client names in multi-client mode.")
+            print("    --gdb     \t Run Carla inside gdb.")
+            print(" -n,--no-gui  \t Run Carla headless, don't show UI.")
             print("")
-            print(" -h,--help   \t Print this help text and exit.")
-            print(" -v,--version\t Print version information and exit.")
+            print(" -h,--help    \t Print this help text and exit.")
+            print(" -v,--version \t Print version information and exit.")
             print("")
 
             sys.exit(0)
@@ -664,20 +685,38 @@ def handleInitialCommandLineArguments(file):
 
             sys.exit(0)
 
+        elif readPrefixNext:
+            readPrefixNext = False
+            gCarla.cnprefix = arg
+
     return (initName, libPrefix)
 
 # ------------------------------------------------------------------------------------------------------------
 # Get initial project file (as passed in the command-line parameters)
 
-def getInitialProjectFile(app, skipExistCheck = False):
-    # FIXME - PyQt mishandles unicode characters, we'll use direct sys.argv for now
+def getInitialProjectFile(skipExistCheck = False):
+    # NOTE: PyQt mishandles unicode characters, we directly use sys.argv instead of qApp->arguments()
     # see https://riverbankcomputing.com/pipermail/pyqt/2015-January/035395.html
-    #args = app.arguments()[1:]
     args = sys.argv[1:]
+    readPrefixNext = False
     for arg in args:
-        if arg.startswith("--with-appname=") or arg.startswith("--with-libprefix=") or arg == "--gdb":
+        if readPrefixNext:
+            readPrefixNext = False
             continue
-        if arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui"):
+        if arg.startswith("--cnprefix="):
+            continue
+        if arg.startswith("--osc-gui="):
+            continue
+        if arg.startswith("--with-appname="):
+            continue
+        if arg.startswith("--with-libprefix="):
+            continue
+        if arg == "--cnprefix":
+            readPrefixNext = True
+            continue
+        if arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui", "--gdb"):
+            continue
+        if MACOS and arg.startswith("-psn_"):
             continue
         arg = os.path.expanduser(arg)
         if skipExistCheck or os.path.exists(arg):
@@ -781,7 +820,7 @@ class QMessageBoxWithBetterWidth(QMessageBox):
 
         lines = self.text().strip().split("\n") + self.informativeText().strip().split("\n")
 
-        if len(lines) > 0:
+        if lines:
             width = 0
 
             for line in lines:
@@ -795,18 +834,19 @@ class QMessageBoxWithBetterWidth(QMessageBox):
 # Safer QSettings class, which does not throw if type mismatches
 
 class QSafeSettings(QSettings):
-    def value(self, key, defaultValue, type):
-        if not isinstance(defaultValue, type):
+    def value(self, key, defaultValue, valueType):
+        if not isinstance(defaultValue, valueType):
             print("QSafeSettings.value() - defaultValue type mismatch for key", key)
 
         try:
-            return QSettings.value(self, key, defaultValue, type)
+            return QSettings.value(self, key, defaultValue, valueType)
         except:
             return defaultValue
 
 # ------------------------------------------------------------------------------------------------------------
 # Custom MessageBox
 
+# pylint: disable=too-many-arguments
 def CustomMessageBox(parent, icon, title, text,
                      extraText="",
                      buttons=QMessageBox.Yes|QMessageBox.No,
@@ -819,5 +859,6 @@ def CustomMessageBox(parent, icon, title, text,
     msgBox.setStandardButtons(buttons)
     msgBox.setDefaultButton(defButton)
     return msgBox.exec_()
+# pylint: enable=too-many-arguments
 
 # ------------------------------------------------------------------------------------------------------------
