@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -20,14 +20,66 @@
   ==============================================================================
 */
 
+#ifndef __clang__
+// GCC4 compatibility
+namespace std { using ::max_align_t; }
+#endif
+
 namespace juce
 {
 
+#ifndef DOXYGEN
+/** The contents of this namespace are used to implement AudioBuffer and should
+    not be used elsewhere. Their interfaces (and existence) are liable to change!
+*/
+namespace detail
+{
+    /** On iOS/arm7 the alignment of `double` is greater than the alignment of
+        `std::max_align_t`, so we can't trust max_align_t. Instead, we query
+        lots of primitive types and use the maximum alignment of all of them.
+
+        We're putting this stuff outside AudioBuffer itself to avoid creating
+        unnecessary copies for each distinct template instantiation of
+        AudioBuffer.
+
+        MSVC 2015 doesn't like when we write getMaxAlignment as a loop which
+        accumulates the max alignment (declarations not allowed in constexpr
+        function body) so instead we use this recursive version which
+        instantiates a zillion templates.
+    */
+
+    template <typename> struct Type {};
+
+    constexpr size_t getMaxAlignment() noexcept { return 0; }
+
+    template <typename Head, typename... Tail>
+    constexpr size_t getMaxAlignment (Type<Head>, Type<Tail>... tail) noexcept
+    {
+        return jmax (alignof (Head), getMaxAlignment (tail...));
+    }
+
+    constexpr size_t maxAlignment = getMaxAlignment (Type<std::max_align_t>{},
+                                                     Type<void*>{},
+                                                     Type<float>{},
+                                                     Type<double>{},
+                                                     Type<long double>{},
+                                                     Type<short int>{},
+                                                     Type<int>{},
+                                                     Type<long int>{},
+                                                     Type<long long int>{},
+                                                     Type<bool>{},
+                                                     Type<char>{},
+                                                     Type<char16_t>{},
+                                                     Type<char32_t>{},
+                                                     Type<wchar_t>{});
+} // namespace detail
+#endif
+
 //==============================================================================
 /**
-    A multi-channel buffer of floating point audio samples.
+    A multi-channel buffer containing floating point audio samples.
 
-    @see AudioSampleBuffer
+    @tags{Audio}
 */
 template <typename Type>
 class AudioBuffer
@@ -36,9 +88,7 @@ public:
     //==============================================================================
     /** Creates an empty buffer with 0 channels and 0 length. */
     AudioBuffer() noexcept
-       : numChannels (0), size (0), allocatedBytes (0),
-         channels (static_cast<Type**> (preallocatedChannelSpace)),
-         isClear (false)
+       : channels (static_cast<Type**> (preallocatedChannelSpace))
     {
     }
 
@@ -80,8 +130,7 @@ public:
                  int numChannelsToUse,
                  int numSamples)
         : numChannels (numChannelsToUse),
-          size (numSamples),
-          allocatedBytes (0)
+          size (numSamples)
     {
         jassert (dataToReferTo != nullptr);
         jassert (numChannelsToUse >= 0 && numSamples >= 0);
@@ -109,9 +158,7 @@ public:
                  int startSample,
                  int numSamples)
         : numChannels (numChannelsToUse),
-          size (numSamples),
-          allocatedBytes (0),
-          isClear (false)
+          size (numSamples)
     {
         jassert (dataToReferTo != nullptr);
         jassert (numChannelsToUse >= 0 && startSample >= 0 && numSamples >= 0);
@@ -121,7 +168,7 @@ public:
     /** Copies another buffer.
 
         This buffer will make its own copy of the other's data, unless the buffer was created
-        using an external data buffer, in which case boths buffers will just point to the same
+        using an external data buffer, in which case both buffers will just point to the same
         shared block of data.
     */
     AudioBuffer (const AudioBuffer& other)
@@ -177,20 +224,22 @@ public:
     /** Destructor.
         This will free any memory allocated by the buffer.
     */
-    ~AudioBuffer() noexcept {}
+    ~AudioBuffer() = default;
 
     /** Move constructor */
     AudioBuffer (AudioBuffer&& other) noexcept
         : numChannels (other.numChannels),
           size (other.size),
           allocatedBytes (other.allocatedBytes),
-          allocatedData (static_cast<HeapBlock<char, true>&&> (other.allocatedData)),
-          isClear (other.isClear)
+          allocatedData (std::move (other.allocatedData)),
+          isClear (other.isClear.load())
     {
         if (numChannels < (int) numElementsInArray (preallocatedChannelSpace))
         {
             channels = preallocatedChannelSpace;
-            memcpy (preallocatedChannelSpace, other.channels, sizeof (preallocatedChannelSpace));
+
+            for (int i = 0; i < numChannels; ++i)
+                preallocatedChannelSpace[i] = other.channels[i];
         }
         else
         {
@@ -208,13 +257,15 @@ public:
         numChannels = other.numChannels;
         size = other.size;
         allocatedBytes = other.allocatedBytes;
-        allocatedData = static_cast<HeapBlock<char, true>&&> (other.allocatedData);
-        isClear = other.isClear;
+        allocatedData = std::move (other.allocatedData);
+        isClear = other.isClear.load();
 
         if (numChannels < (int) numElementsInArray (preallocatedChannelSpace))
         {
             channels = preallocatedChannelSpace;
-            memcpy (preallocatedChannelSpace, other.channels, sizeof (preallocatedChannelSpace));
+
+            for (int i = 0; i < numChannels; ++i)
+                preallocatedChannelSpace[i] = other.channels[i];
         }
         else
         {
@@ -324,6 +375,9 @@ public:
         a new allocation will be done so that the buffer uses takes up the minimum amount
         of memory that it needs.
 
+        Note that if keepExistingContent and avoidReallocating are both true, then it will
+        only avoid reallocating if neither the channel count or length in samples increase.
+
         If the required memory can't be allocated, this will throw a std::bad_alloc exception.
     */
     void setSize (int newNumChannels,
@@ -337,38 +391,45 @@ public:
 
         if (newNumSamples != size || newNumChannels != numChannels)
         {
-            const auto allocatedSamplesPerChannel = ((size_t) newNumSamples + 3) & ~3u;
-            const auto channelListSize = ((sizeof (Type*) * (size_t) (newNumChannels + 1)) + 15) & ~15u;
-            const auto newTotalBytes = ((size_t) newNumChannels * (size_t) allocatedSamplesPerChannel * sizeof (Type))
-                                          + channelListSize + 32;
+            auto allocatedSamplesPerChannel = ((size_t) newNumSamples + 3) & ~3u;
+            auto channelListSize = ((static_cast<size_t> (1 + newNumChannels) * sizeof (Type*)) + 15) & ~15u;
+            auto newTotalBytes = ((size_t) newNumChannels * (size_t) allocatedSamplesPerChannel * sizeof (Type))
+                                    + channelListSize + 32;
 
             if (keepExistingContent)
             {
-                HeapBlock<char, true> newData;
-                newData.allocate (newTotalBytes, clearExtraSpace || isClear);
-
-                auto numSamplesToCopy = (size_t) jmin (newNumSamples, size);
-
-                auto newChannels = reinterpret_cast<Type**> (newData.get());
-                auto newChan     = reinterpret_cast<Type*> (newData + channelListSize);
-
-                for (int j = 0; j < newNumChannels; ++j)
+                if (avoidReallocating && newNumChannels <= numChannels && newNumSamples <= size)
                 {
-                    newChannels[j] = newChan;
-                    newChan += allocatedSamplesPerChannel;
+                    // no need to do any remapping in this case, as the channel pointers will remain correct!
                 }
-
-                if (! isClear)
+                else
                 {
-                    auto numChansToCopy = jmin (numChannels, newNumChannels);
+                    HeapBlock<char, true> newData;
+                    newData.allocate (newTotalBytes, clearExtraSpace || isClear);
 
-                    for (int i = 0; i < numChansToCopy; ++i)
-                        FloatVectorOperations::copy (newChannels[i], channels[i], (int) numSamplesToCopy);
+                    auto numSamplesToCopy = (size_t) jmin (newNumSamples, size);
+
+                    auto newChannels = reinterpret_cast<Type**> (newData.get());
+                    auto newChan     = reinterpret_cast<Type*> (newData + channelListSize);
+
+                    for (int j = 0; j < newNumChannels; ++j)
+                    {
+                        newChannels[j] = newChan;
+                        newChan += allocatedSamplesPerChannel;
+                    }
+
+                    if (! isClear)
+                    {
+                        auto numChansToCopy = jmin (numChannels, newNumChannels);
+
+                        for (int i = 0; i < numChansToCopy; ++i)
+                            FloatVectorOperations::copy (newChannels[i], channels[i], (int) numSamplesToCopy);
+                    }
+
+                    allocatedData.swapWith (newData);
+                    allocatedBytes = newTotalBytes;
+                    channels = newChannels;
                 }
-
-                allocatedData.swapWith (newData);
-                allocatedBytes = newTotalBytes;
-                channels = newChannels;
             }
             else
             {
@@ -393,7 +454,7 @@ public:
                 }
             }
 
-            channels[newNumChannels] = 0;
+            channels[newNumChannels] = nullptr;
             size = newNumSamples;
             numChannels = newNumChannels;
         }
@@ -595,11 +656,11 @@ public:
         jassert (isPositiveAndBelow (channel, numChannels));
         jassert (startSample >= 0 && numSamples >= 0 && startSample + numSamples <= size);
 
-        if (gain != (Type) 1 && ! isClear)
+        if (gain != Type (1) && ! isClear)
         {
             auto* d = channels[channel] + startSample;
 
-            if (gain == 0)
+            if (gain == Type())
                 FloatVectorOperations::clear (d, numSamples);
             else
                 FloatVectorOperations::multiply (d, gain, numSamples);
@@ -693,7 +754,7 @@ public:
                   int sourceChannel,
                   int sourceStartSample,
                   int numSamples,
-                  Type gainToApplyToSource = (Type) 1) noexcept
+                  Type gainToApplyToSource = Type (1)) noexcept
     {
         jassert (&source != this || sourceChannel != destChannel);
         jassert (isPositiveAndBelow (destChannel, numChannels));
@@ -710,14 +771,14 @@ public:
             {
                 isClear = false;
 
-                if (gainToApplyToSource != (Type) 1)
+                if (gainToApplyToSource != Type (1))
                     FloatVectorOperations::copyWithMultiply (d, s, gainToApplyToSource, numSamples);
                 else
                     FloatVectorOperations::copy (d, s, numSamples);
             }
             else
             {
-                if (gainToApplyToSource != (Type) 1)
+                if (gainToApplyToSource != Type (1))
                     FloatVectorOperations::addWithMultiply (d, s, gainToApplyToSource, numSamples);
                 else
                     FloatVectorOperations::add (d, s, numSamples);
@@ -725,7 +786,6 @@ public:
         }
     }
 
-
     /** Adds samples from an array of floats to one of the channels.
 
         @param destChannel          the channel within this buffer to add the samples to
@@ -741,7 +801,7 @@ public:
                   int destStartSample,
                   const Type* source,
                   int numSamples,
-                  Type gainToApplyToSource = (Type) 1) noexcept
+                  Type gainToApplyToSource = Type (1)) noexcept
     {
         jassert (isPositiveAndBelow (destChannel, numChannels));
         jassert (destStartSample >= 0 && numSamples >= 0 && destStartSample + numSamples <= size);
@@ -755,14 +815,14 @@ public:
             {
                 isClear = false;
 
-                if (gainToApplyToSource != (Type) 1)
+                if (gainToApplyToSource != Type (1))
                     FloatVectorOperations::copyWithMultiply (d, source, gainToApplyToSource, numSamples);
                 else
                     FloatVectorOperations::copy (d, source, numSamples);
             }
             else
             {
-                if (gainToApplyToSource != (Type) 1)
+                if (gainToApplyToSource != Type (1))
                     FloatVectorOperations::addWithMultiply (d, source, gainToApplyToSource, numSamples);
                 else
                     FloatVectorOperations::add (d, source, numSamples);
@@ -904,9 +964,9 @@ public:
         {
             auto* d = channels[destChannel] + destStartSample;
 
-            if (gain != (Type) 1)
+            if (gain != Type (1))
             {
-                if (gain == 0)
+                if (gain == Type())
                 {
                     if (! isClear)
                         FloatVectorOperations::clear (d, numSamples);
@@ -982,7 +1042,7 @@ public:
         jassert (startSample >= 0 && numSamples >= 0 && startSample + numSamples <= size);
 
         if (isClear)
-            return {};
+            return { Type (0), Type (0) };
 
         return FloatVectorOperations::findMinAndMax (channels[channel] + startSample, numSamples);
     }
@@ -994,7 +1054,7 @@ public:
         jassert (startSample >= 0 && numSamples >= 0 && startSample + numSamples <= size);
 
         if (isClear)
-            return {};
+            return Type (0);
 
         auto r = findMinMax (channel, startSample, numSamples);
 
@@ -1004,7 +1064,7 @@ public:
     /** Finds the highest absolute sample value within a region on all channels. */
     Type getMagnitude (int startSample, int numSamples) const noexcept
     {
-        Type mag = 0;
+        Type mag (0);
 
         if (! isClear)
             for (int i = 0; i < numChannels; ++i)
@@ -1020,18 +1080,18 @@ public:
         jassert (startSample >= 0 && numSamples >= 0 && startSample + numSamples <= size);
 
         if (numSamples <= 0 || channel < 0 || channel >= numChannels || isClear)
-            return {};
+            return Type (0);
 
         auto* data = channels[channel] + startSample;
         double sum = 0.0;
 
         for (int i = 0; i < numSamples; ++i)
         {
-            const Type sample = data[i];
+            auto sample = data[i];
             sum += sample * sample;
         }
 
-        return (Type) std::sqrt (sum / numSamples);
+        return static_cast<Type> (std::sqrt (sum / numSamples));
     }
 
     /** Reverses a part of a channel. */
@@ -1052,24 +1112,38 @@ public:
             reverse (i, startSample, numSamples);
     }
 
+    //==============================================================================
+    /** This allows templated code that takes an AudioBuffer to access its sample type. */
+    using SampleType = Type;
 
 private:
     //==============================================================================
-    int numChannels, size;
-    size_t allocatedBytes;
+    int numChannels = 0, size = 0;
+    size_t allocatedBytes = 0;
     Type** channels;
     HeapBlock<char, true> allocatedData;
     Type* preallocatedChannelSpace[32];
-    bool isClear;
+    std::atomic<bool> isClear { false };
 
     void allocateData()
     {
+       #if (! JUCE_GCC) || (__GNUC__ * 100 + __GNUC_MINOR__) >= 409
+        static_assert (alignof (Type) <= detail::maxAlignment,
+                       "AudioBuffer cannot hold types with alignment requirements larger than that guaranteed by malloc");
+       #endif
         jassert (size >= 0);
-        auto channelListSize = sizeof (Type*) * (size_t) (numChannels + 1);
+
+        auto channelListSize = (size_t) (numChannels + 1) * sizeof (Type*);
+        auto requiredSampleAlignment = std::alignment_of<Type>::value;
+        size_t alignmentOverflow = channelListSize % requiredSampleAlignment;
+
+        if (alignmentOverflow != 0)
+            channelListSize += requiredSampleAlignment - alignmentOverflow;
+
         allocatedBytes = (size_t) numChannels * (size_t) size * sizeof (Type) + channelListSize + 32;
         allocatedData.malloc (allocatedBytes);
         channels = reinterpret_cast<Type**> (allocatedData.get());
-        auto* chan = (Type*) (allocatedData + channelListSize);
+        auto chan = reinterpret_cast<Type*> (allocatedData + channelListSize);
 
         for (int i = 0; i < numChannels; ++i)
         {
@@ -1092,7 +1166,7 @@ private:
         }
         else
         {
-            allocatedData.malloc ((size_t) numChannels + 1, sizeof (Type*));
+            allocatedData.malloc (numChannels + 1, sizeof (Type*));
             channels = reinterpret_cast<Type**> (allocatedData.get());
         }
 
@@ -1100,7 +1174,6 @@ private:
         {
             // you have to pass in the same number of valid pointers as numChannels
             jassert (dataToReferTo[i] != nullptr);
-
             channels[i] = dataToReferTo[i] + offset;
         }
 
@@ -1115,12 +1188,12 @@ private:
 /**
     A multi-channel buffer of 32-bit floating point audio samples.
 
-    This typedef is here for backwards compatibility with the older AudioSampleBuffer
+    This type is here for backwards compatibility with the older AudioSampleBuffer
     class, which was fixed for 32-bit data, but is otherwise the same as the new
     templated AudioBuffer class.
 
     @see AudioBuffer
 */
-typedef AudioBuffer<float> AudioSampleBuffer;
+using AudioSampleBuffer = AudioBuffer<float>;
 
 } // namespace juce
