@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -43,7 +43,9 @@ namespace ASIODebugging
     {
         message = "ASIO: " + message;
         DBG (message);
-        Logger::writeToLog (message);
+
+        if (Logger::getCurrentLogger() != nullptr)
+            Logger::writeToLog (message);
     }
 
     static void logError (const String& context, long error)
@@ -77,11 +79,7 @@ struct ASIOSampleFormat
 {
     ASIOSampleFormat() noexcept {}
 
-    ASIOSampleFormat (const long type) noexcept
-        : bitDepth (24),
-          byteStride (4),
-          formatIsFloat (false),
-          littleEndian (true)
+    ASIOSampleFormat (long type) noexcept
     {
         switch (type)
         {
@@ -114,7 +112,7 @@ struct ASIOSampleFormat
         }
     }
 
-    void convertToFloat (const void* const src, float* const dst, const int samps) const noexcept
+    void convertToFloat (const void* src, float* dst, int samps) const noexcept
     {
         if (formatIsFloat)
         {
@@ -132,7 +130,7 @@ struct ASIOSampleFormat
         }
     }
 
-    void convertFromFloat (const float* const src, void* const dst, const int samps) const noexcept
+    void convertFromFloat (const float* src, void* dst, int samps) const noexcept
     {
         if (formatIsFloat)
         {
@@ -150,18 +148,18 @@ struct ASIOSampleFormat
         }
     }
 
-    void clear (void* dst, const int numSamps) noexcept
+    void clear (void* dst, int numSamps) noexcept
     {
         if (dst != nullptr)
             zeromem (dst, numSamps * byteStride);
     }
 
-    int bitDepth, byteStride;
-    bool formatIsFloat, littleEndian;
+    int bitDepth = 24, byteStride = 4;
+    bool formatIsFloat = false, littleEndian = true;
 
 private:
-    static void convertInt16ToFloat (const char* src, float* dest, const int srcStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertInt16ToFloat (const char* src, float* dest, int srcStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double g = 1.0 / 32768.0;
 
@@ -183,8 +181,8 @@ private:
         }
     }
 
-    static void convertFloatToInt16 (const float* src, char* dest, const int dstStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertFloatToInt16 (const float* src, char* dest, int dstStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double maxVal = (double) 0x7fff;
 
@@ -206,8 +204,8 @@ private:
         }
     }
 
-    static void convertInt24ToFloat (const char* src, float* dest, const int srcStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertInt24ToFloat (const char* src, float* dest, int srcStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double g = 1.0 / 0x7fffff;
 
@@ -229,8 +227,8 @@ private:
         }
     }
 
-    static void convertFloatToInt24 (const float* src, char* dest, const int dstStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertFloatToInt24 (const float* src, char* dest, int dstStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double maxVal = (double) 0x7fffff;
 
@@ -252,8 +250,8 @@ private:
         }
     }
 
-    static void convertInt32ToFloat (const char* src, float* dest, const int srcStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertInt32ToFloat (const char* src, float* dest, int srcStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double g = 1.0 / 0x7fffffff;
 
@@ -275,8 +273,8 @@ private:
         }
     }
 
-    static void convertFloatToInt32 (const float* src, char* dest, const int dstStrideBytes,
-                                     int numSamples, const bool littleEndian) noexcept
+    static void convertFloatToInt32 (const float* src, char* dest, int dstStrideBytes,
+                                     int numSamples, bool littleEndian) noexcept
     {
         const double maxVal = (double) 0x7fffffff;
 
@@ -300,8 +298,9 @@ private:
 };
 
 //==============================================================================
+constexpr int maxNumASIODevices = 16;
 class ASIOAudioIODevice;
-static ASIOAudioIODevice* volatile currentASIODev[16] = { 0 };
+static ASIOAudioIODevice* currentASIODev[maxNumASIODevices] = {};
 
 extern HWND juce_messageWindowHandle;
 
@@ -314,33 +313,10 @@ class ASIOAudioIODevice  : public AudioIODevice,
 {
 public:
     ASIOAudioIODevice (ASIOAudioIODeviceType* ownerType, const String& devName,
-                       const CLSID clsID, const int slotNumber)
+                       CLSID clsID, int slotNumber)
        : AudioIODevice (devName, "ASIO"),
          owner (ownerType),
-         asioObject (nullptr),
-         classId (clsID),
-         inputLatency (0),
-         outputLatency (0),
-         minBufferSize (0), maxBufferSize (0),
-         preferredBufferSize (0),
-         bufferGranularity (0),
-         numClockSources (0),
-         currentBlockSizeSamples (0),
-         currentBitDepth (16),
-         currentSampleRate (0),
-         currentCallback (nullptr),
-         bufferIndex (0),
-         numActiveInputChans (0),
-         numActiveOutputChans (0),
-         deviceIsOpen (false),
-         isStarted (false),
-         buffersCreated (false),
-         calledback (false),
-         littleEndian (false),
-         postOutput (true),
-         needToReset (false),
-         insideControlPanelModalLoop (false),
-         shouldUsePreferredSize (false)
+         classId (clsID)
     {
         ::CoInitialize (nullptr);
 
@@ -348,21 +324,23 @@ public:
         inBuffers.calloc (4);
         outBuffers.calloc (4);
 
-        jassert (currentASIODev [slotNumber] == nullptr);
-        currentASIODev [slotNumber] = this;
+        jassert (currentASIODev[slotNumber] == nullptr);
+        currentASIODev[slotNumber] = this;
 
         openDevice();
     }
 
     ~ASIOAudioIODevice()
     {
-        for (int i = 0; i < numElementsInArray (currentASIODev); ++i)
+        for (int i = 0; i < maxNumASIODevices; ++i)
             if (currentASIODev[i] == this)
                 currentASIODev[i] = nullptr;
 
         close();
         JUCE_ASIO_LOG ("closed");
-        removeCurrentDriver();
+
+        if (! removeCurrentDriver())
+            JUCE_ASIO_LOG ("** Driver crashed while being closed");
     }
 
     void updateSampleRates()
@@ -372,16 +350,16 @@ public:
 
         if (asioObject != nullptr)
         {
-            const int possibleSampleRates[] = { 32000, 44100, 48000, 88200, 96000, 176400, 192000, 352800, 384000 };
-
-            for (int index = 0; index < numElementsInArray (possibleSampleRates); ++index)
-                if (asioObject->canSampleRate ((double) possibleSampleRates[index]) == 0)
-                    newRates.add ((double) possibleSampleRates[index]);
+            for (auto rate : { 8000, 11025, 16000, 22050, 32000,
+                               44100, 48000, 88200, 96000, 176400,
+                               192000, 352800, 384000, 705600, 768000 })
+                if (asioObject->canSampleRate ((double) rate) == 0)
+                    newRates.add ((double) rate);
         }
 
         if (newRates.isEmpty())
         {
-            double cr = getSampleRate();
+            auto cr = getSampleRate();
             JUCE_ASIO_LOG ("No sample rates supported - current rate: " + String ((int) cr));
 
             if (cr > 0)
@@ -394,8 +372,9 @@ public:
 
            #if JUCE_ASIO_DEBUGGING
             StringArray s;
-            for (int i = 0; i < sampleRates.size(); ++i)
-                s.add (String (sampleRates.getUnchecked(i)));
+
+            for (auto r : sampleRates)
+                s.add (String (r));
 
             JUCE_ASIO_LOG ("Rates: " + s.joinIntoString (" "));
            #endif
@@ -425,21 +404,20 @@ public:
 
         if (asioObject == nullptr)
         {
-            const String openingError (openDevice());
+            auto openingError = openDevice();
 
             if (asioObject == nullptr)
                 return openingError;
         }
 
         isStarted = false;
-        bufferIndex = -1;
 
-        long err = asioObject->getChannels (&totalNumInputChans, &totalNumOutputChans);
+        auto err = asioObject->getChannels (&totalNumInputChans, &totalNumOutputChans);
         jassert (err == ASE_OK);
 
         bufferSizeSamples = readBufferSizes (bufferSizeSamples);
 
-        double sampleRate = sr;
+        auto sampleRate = sr;
         currentSampleRate = sampleRate;
         currentBlockSizeSamples = bufferSizeSamples;
         currentChansOut.clear();
@@ -450,9 +428,11 @@ public:
         if (sampleRate == 0 || (sampleRates.size() > 0 && ! sampleRates.contains (sampleRate)))
             sampleRate = sampleRates[0];
 
-        jassert (sampleRate != 0);
         if (sampleRate == 0)
+        {
+            jassertfalse;
             sampleRate = 44100.0;
+        }
 
         updateClockSources();
         currentSampleRate = getSampleRate();
@@ -475,7 +455,9 @@ public:
         if (needToReset)
         {
             JUCE_ASIO_LOG (" Resetting");
-            removeCurrentDriver();
+
+            if (! removeCurrentDriver())
+                JUCE_ASIO_LOG ("** Driver crashed while being closed");
 
             loadDriver();
             String initError = initDriver();
@@ -483,10 +465,12 @@ public:
             if (initError.isNotEmpty())
                 JUCE_ASIO_LOG ("ASIOInit: " + initError);
 
+            setSampleRate (getSampleRate());
+
             needToReset = false;
         }
 
-        const int totalBuffers = resetBuffers (inputChannels, outputChannels);
+        auto totalBuffers = resetBuffers (inputChannels, outputChannels);
 
         setCallbackFunctions();
 
@@ -508,18 +492,17 @@ public:
         if (err == ASE_OK)
         {
             buffersCreated = true;
-
-            tempBuffer.calloc (totalBuffers * currentBlockSizeSamples + 32);
+            ioBufferSpace.calloc (totalBuffers * currentBlockSizeSamples + 32);
 
             int n = 0;
-            Array <int> types;
+            Array<int> types;
             currentBitDepth = 16;
 
             for (int i = 0; i < (int) totalNumInputChans; ++i)
             {
                 if (inputChannels[i])
                 {
-                    inBuffers[n] = tempBuffer + (currentBlockSizeSamples * n);
+                    inBuffers[n] = ioBufferSpace + (currentBlockSizeSamples * n);
 
                     ASIOChannelInfo channelInfo = { 0 };
                     channelInfo.channel = i;
@@ -541,7 +524,7 @@ public:
             {
                 if (outputChannels[i])
                 {
-                    outBuffers[n] = tempBuffer + (currentBlockSizeSamples * (numActiveInputChans + n));
+                    outBuffers[n] = ioBufferSpace + (currentBlockSizeSamples * (numActiveInputChans + n));
 
                     ASIOChannelInfo channelInfo = { 0 };
                     channelInfo.channel = i;
@@ -565,8 +548,8 @@ public:
 
             for (int i = 0; i < numActiveOutputChans; ++i)
             {
-                outputFormat[i].clear (bufferInfos [numActiveInputChans + i].buffers[0], currentBlockSizeSamples);
-                outputFormat[i].clear (bufferInfos [numActiveInputChans + i].buffers[1], currentBlockSizeSamples);
+                outputFormat[i].clear (bufferInfos[numActiveInputChans + i].buffers[0], currentBlockSizeSamples);
+                outputFormat[i].clear (bufferInfos[numActiveInputChans + i].buffers[1], currentBlockSizeSamples);
             }
 
             readLatencies();
@@ -616,7 +599,7 @@ public:
             isStarted = false;
             deviceIsOpen = false;
 
-            const String errorCopy (error);
+            auto errorCopy = error;
             close(); // (this resets the error string)
             error = errorCopy;
         }
@@ -663,8 +646,8 @@ public:
     BigInteger getActiveOutputChannels() const override    { return currentChansOut; }
     BigInteger getActiveInputChannels() const override     { return currentChansIn; }
 
-    int getOutputLatencyInSamples() override     { return outputLatency + currentBlockSizeSamples / 4; }
-    int getInputLatencyInSamples() override      { return inputLatency + currentBlockSizeSamples / 4; }
+    int getOutputLatencyInSamples() override     { return outputLatency; }
+    int getInputLatencyInSamples() override      { return inputLatency; }
 
     void start (AudioIODeviceCallback* callback) override
     {
@@ -679,7 +662,7 @@ public:
 
     void stop() override
     {
-        AudioIODeviceCallback* const lastCallback = currentCallback;
+        auto* lastCallback = currentCallback;
 
         {
             const ScopedLock sl (callbackLock);
@@ -699,15 +682,13 @@ public:
 
         bool done = false;
         insideControlPanelModalLoop = true;
-
-        const uint32 started = Time::getMillisecondCounter();
+        auto started = Time::getMillisecondCounter();
 
         if (asioObject != nullptr)
         {
             asioObject->controlPanel();
 
-            const int spent = (int) Time::getMillisecondCounter() - (int) started;
-
+            auto spent = (int) (Time::getMillisecondCounter() - started);
             JUCE_ASIO_LOG ("spent: " + String (spent));
 
             if (spent > 300)
@@ -731,13 +712,10 @@ public:
         if (! insideControlPanelModalLoop)
         {
             stopTimer();
-
             JUCE_ASIO_LOG ("restart request!");
 
-            AudioIODeviceCallback* const oldCallback = currentCallback;
-
+            auto* oldCallback = currentCallback;
             close();
-
             needToReset = true;
             open (BigInteger (currentChansIn), BigInteger (currentChansOut),
                   currentSampleRate, currentBlockSizeSamples);
@@ -758,51 +736,49 @@ public:
 private:
     //==============================================================================
     WeakReference<ASIOAudioIODeviceType> owner;
-    IASIO* volatile asioObject;
+    IASIO* asioObject = {};
     ASIOCallbacks callbacks;
 
     CLSID classId;
     String error;
 
-    long totalNumInputChans, totalNumOutputChans;
+    long totalNumInputChans = 0, totalNumOutputChans = 0;
     StringArray inputChannelNames, outputChannelNames;
 
     Array<double> sampleRates;
     Array<int> bufferSizes;
-    long inputLatency, outputLatency;
-    long minBufferSize, maxBufferSize, preferredBufferSize, bufferGranularity;
-    ASIOClockSource clocks[32];
-    int numClockSources;
+    long inputLatency = 0, outputLatency = 0;
+    long minBufferSize = 0, maxBufferSize = 0, preferredBufferSize = 0, bufferGranularity = 0;
+    ASIOClockSource clocks[32] = {};
+    int numClockSources = 0;
 
-    int volatile currentBlockSizeSamples;
-    int volatile currentBitDepth;
-    double volatile currentSampleRate;
+    int currentBlockSizeSamples = 0;
+    int currentBitDepth = 16;
+    double currentSampleRate = 0;
     BigInteger currentChansOut, currentChansIn;
-    AudioIODeviceCallback* volatile currentCallback;
+    AudioIODeviceCallback* currentCallback = {};
     CriticalSection callbackLock;
 
     HeapBlock<ASIOBufferInfo> bufferInfos;
     HeapBlock<float*> inBuffers, outBuffers;
+    HeapBlock<float> ioBufferSpace;
     HeapBlock<ASIOSampleFormat> inputFormat, outputFormat;
+    int numActiveInputChans = 0, numActiveOutputChans = 0;
 
-    WaitableEvent event1;
-    HeapBlock<float> tempBuffer;
-    int volatile bufferIndex, numActiveInputChans, numActiveOutputChans;
-
-    bool deviceIsOpen, isStarted, buffersCreated;
-    bool volatile calledback;
-    bool volatile littleEndian, postOutput, needToReset;
-    bool volatile insideControlPanelModalLoop;
-    bool volatile shouldUsePreferredSize;
+    bool deviceIsOpen = false, isStarted = false, buffersCreated = false;
+    std::atomic<bool> calledback { false };
+    bool littleEndian = false, postOutput = true, needToReset = false;
+    bool insideControlPanelModalLoop = false;
+    bool shouldUsePreferredSize = false;
     int xruns = 0;
 
     //==============================================================================
-    static String convertASIOString (char* const text, int length)
+    static String convertASIOString (char* text, int length)
     {
         if (CharPointer_UTF8::isValidString (text, length))
             return String::fromUTF8 (text, length);
 
-        WCHAR wideVersion [64] = { 0 };
+        WCHAR wideVersion[512] = {};
         MultiByteToWideChar (CP_ACP, 0, text, length, wideVersion, numElementsInArray (wideVersion));
         return wideVersion;
     }
@@ -819,9 +795,14 @@ private:
 
     void reloadChannelNames()
     {
+        long totalInChannels = 0, totalOutChannels = 0;
+
         if (asioObject != nullptr
-             && asioObject->getChannels (&totalNumInputChans, &totalNumOutputChans) == ASE_OK)
+             && asioObject->getChannels (&totalInChannels, &totalOutChannels) == ASE_OK)
         {
+            totalNumInputChans  = totalInChannels;
+            totalNumOutputChans = totalOutChannels;
+
             inputChannelNames.clear();
             outputChannelNames.clear();
 
@@ -868,7 +849,7 @@ private:
         if (shouldUsePreferredSize)
         {
             JUCE_ASIO_LOG ("Using preferred size for buffer..");
-            long err = refreshBufferSizes();
+            auto err = refreshBufferSizes();
 
             if (err == ASE_OK)
             {
@@ -891,8 +872,8 @@ private:
     {
         numActiveInputChans = 0;
         numActiveOutputChans = 0;
+        auto* info = bufferInfos.get();
 
-        ASIOBufferInfo* info = bufferInfos;
         for (int i = 0; i < totalNumInputChans; ++i)
         {
             if (inputChannels[i])
@@ -953,7 +934,7 @@ private:
     double getSampleRate() const
     {
         double cr = 0;
-        long err = asioObject->getSampleRate (&cr);
+        auto err = asioObject->getSampleRate (&cr);
         JUCE_ASIO_LOG_ERROR ("getSampleRate", err);
         return cr;
     }
@@ -963,7 +944,7 @@ private:
         if (currentSampleRate != newRate)
         {
             JUCE_ASIO_LOG ("rate change: " + String (currentSampleRate) + " to " + String (newRate));
-            long err = asioObject->setSampleRate (newRate);
+            auto err = asioObject->setSampleRate (newRate);
 
             if (err == ASE_NoClock && numClockSources > 0)
             {
@@ -971,7 +952,6 @@ private:
                 Thread::sleep (10);
                 err = asioObject->setClockSource (clocks[0].index);
                 JUCE_ASIO_LOG_ERROR ("setClockSource2", err);
-
                 Thread::sleep (10);
                 err = asioObject->setSampleRate (newRate);
             }
@@ -1010,7 +990,7 @@ private:
         if (numClockSources > 1 && ! isSourceSet)
         {
             JUCE_ASIO_LOG ("setting clock source");
-            long err = asioObject->setClockSource (clocks[0].index);
+            auto err = asioObject->setClockSource (clocks[0].index);
             JUCE_ASIO_LOG_ERROR ("setClockSource1", err);
             Thread::sleep (20);
         }
@@ -1036,7 +1016,7 @@ private:
         numActiveInputChans = 0;
         numActiveOutputChans = 0;
 
-        ASIOBufferInfo* info = bufferInfos;
+        auto* info = bufferInfos.get();
         int numChans = 0;
 
         for (int i = 0; i < jmin (2, (int) totalNumInputChans); ++i)
@@ -1065,7 +1045,7 @@ private:
 
         if (preferredSize > 0)
         {
-            long err = asioObject->createBuffers (bufferInfos, numChans, preferredSize, &callbacks);
+            auto err = asioObject->createBuffers (bufferInfos, numChans, preferredSize, &callbacks);
             JUCE_ASIO_LOG_ERROR ("dummy buffers", err);
         }
 
@@ -1095,24 +1075,38 @@ private:
             if (i < 2)
             {
                 // clear the channels that are used with the dummy stuff
-                outputFormat[i].clear (bufferInfos [outputBufferIndex + i].buffers[0], preferredBufferSize);
-                outputFormat[i].clear (bufferInfos [outputBufferIndex + i].buffers[1], preferredBufferSize);
+                outputFormat[i].clear (bufferInfos[outputBufferIndex + i].buffers[0], preferredBufferSize);
+                outputFormat[i].clear (bufferInfos[outputBufferIndex + i].buffers[1], preferredBufferSize);
             }
         }
     }
 
-    void removeCurrentDriver()
+    bool removeCurrentDriver()
     {
+        bool releasedOK = true;
+
         if (asioObject != nullptr)
         {
-            asioObject->Release();
+           #if ! JUCE_MINGW
+            __try
+           #endif
+            {
+                asioObject->Release();
+            }
+           #if ! JUCE_MINGW
+            __except (EXCEPTION_EXECUTE_HANDLER) { releasedOK = false; }
+           #endif
+
             asioObject = nullptr;
         }
+
+        return releasedOK;
     }
 
     bool loadDriver()
     {
-        removeCurrentDriver();
+        if (! removeCurrentDriver())
+            JUCE_ASIO_LOG ("** Driver crashed while being closed");
 
         bool crashed = false;
         bool ok = tryCreatingDriver (crashed);
@@ -1132,9 +1126,7 @@ private:
             return CoCreateInstance (classId, 0, CLSCTX_INPROC_SERVER,
                                      classId, (void**) &asioObject) == S_OK;
         }
-       #if JUCE_MINGW
-        ignoreUnused(crashed);
-       #else
+       #if ! JUCE_MINGW
         __except (EXCEPTION_EXECUTE_HANDLER) { crashed = true; }
         return false;
        #endif
@@ -1143,9 +1135,11 @@ private:
     String getLastDriverError() const
     {
         jassert (asioObject != nullptr);
-        char buffer [512] = { 0 };
+
+        char buffer[512] = {};
         asioObject->getErrorMessage (buffer);
-        return String (buffer, sizeof (buffer) - 1);
+
+        return convertASIOString (buffer, sizeof (buffer));
     }
 
     String initDriver()
@@ -1153,7 +1147,7 @@ private:
         if (asioObject == nullptr)
             return "No Driver";
 
-        const bool initOk = !! asioObject->init (juce_messageWindowHandle);
+        auto initOk = (asioObject->init (juce_messageWindowHandle) > 0);
         String driverError;
 
         // Get error message if init() failed, or if it's a buggy Denon driver,
@@ -1166,7 +1160,7 @@ private:
 
         if (driverError.isEmpty())
         {
-            char buffer [512];
+            char buffer[512] = {};
             asioObject->getDriverName (buffer); // just in case any flimsy drivers expect this to be called..
         }
 
@@ -1190,7 +1184,6 @@ private:
         numActiveOutputChans = 0;
         xruns = 0;
         currentCallback = nullptr;
-
         error.clear();
 
         if (getName().isEmpty())
@@ -1223,7 +1216,7 @@ private:
                     {
                         addBufferSizes (minBufferSize, maxBufferSize, preferredBufferSize, bufferGranularity);
 
-                        double currentRate = getSampleRate();
+                        auto currentRate = getSampleRate();
 
                         if (currentRate < 1.0 || currentRate > 192001.0)
                         {
@@ -1235,13 +1228,12 @@ private:
                         }
 
                         currentSampleRate = currentRate;
-
                         postOutput = (asioObject->outputReady() == 0);
+
                         if (postOutput)
                             JUCE_ASIO_LOG ("outputReady true");
 
                         updateSampleRates();
-
                         readLatencies();                          // ..doing these steps because cubase does so at this stage
                         createDummyBuffers (preferredBufferSize); // in initialisation, and some devices fail if we don't.
                         readLatencies();
@@ -1274,7 +1266,9 @@ private:
         {
             JUCE_ASIO_LOG_ERROR (error, err);
             disposeBuffers();
-            removeCurrentDriver();
+
+            if (! removeCurrentDriver())
+                JUCE_ASIO_LOG ("** Driver crashed while being closed");
         }
         else
         {
@@ -1297,12 +1291,11 @@ private:
     }
 
     //==============================================================================
-    void JUCE_ASIOCALLBACK callback (const long index)
+    void JUCE_ASIOCALLBACK callback (long index)
     {
         if (isStarted)
         {
-            bufferIndex = index;
-            processBuffer();
+            processBuffer (index);
         }
         else
         {
@@ -1313,23 +1306,21 @@ private:
         calledback = true;
     }
 
-    void processBuffer()
+    void processBuffer (long bufferIndex)
     {
-        const ASIOBufferInfo* const infos = bufferInfos;
-        const int bi = bufferIndex;
-
         const ScopedLock sl (callbackLock);
 
-        if (bi >= 0)
+        if (bufferIndex >= 0)
         {
-            const int samps = currentBlockSizeSamples;
+            auto* infos = bufferInfos.get();
+            auto samps = currentBlockSizeSamples;
 
             if (currentCallback != nullptr)
             {
                 for (int i = 0; i < numActiveInputChans; ++i)
                 {
                     jassert (inBuffers[i] != nullptr);
-                    inputFormat[i].convertToFloat (infos[i].buffers[bi], inBuffers[i], samps);
+                    inputFormat[i].convertToFloat (infos[i].buffers[bufferIndex], inBuffers[i], samps);
                 }
 
                 currentCallback->audioDeviceIOCallback (const_cast<const float**> (inBuffers.getData()), numActiveInputChans,
@@ -1338,13 +1329,13 @@ private:
                 for (int i = 0; i < numActiveOutputChans; ++i)
                 {
                     jassert (outBuffers[i] != nullptr);
-                    outputFormat[i].convertFromFloat (outBuffers[i], infos [numActiveInputChans + i].buffers[bi], samps);
+                    outputFormat[i].convertFromFloat (outBuffers[i], infos[numActiveInputChans + i].buffers[bufferIndex], samps);
                 }
             }
             else
             {
                 for (int i = 0; i < numActiveOutputChans; ++i)
-                     outputFormat[i].clear (infos[numActiveInputChans + i].buffers[bi], samps);
+                     outputFormat[i].clear (infos[numActiveInputChans + i].buffers[bufferIndex], samps);
             }
         }
 
@@ -1362,15 +1353,15 @@ private:
                     return 1;
                 break;
 
-            case kAsioBufferSizeChange: JUCE_ASIO_LOG ("kAsioBufferSizeChange"); resetRequest(); return 1;
-            case kAsioResetRequest:     JUCE_ASIO_LOG ("kAsioResetRequest");     resetRequest(); return 1;
-            case kAsioResyncRequest:    JUCE_ASIO_LOG ("kAsioResyncRequest");    resetRequest(); return 1;
-            case kAsioLatenciesChanged: JUCE_ASIO_LOG ("kAsioLatenciesChanged"); return 1;
-            case kAsioEngineVersion:    return 2;
+            case kAsioBufferSizeChange:  JUCE_ASIO_LOG ("kAsioBufferSizeChange"); resetRequest(); return 1;
+            case kAsioResetRequest:      JUCE_ASIO_LOG ("kAsioResetRequest");     resetRequest(); return 1;
+            case kAsioResyncRequest:     JUCE_ASIO_LOG ("kAsioResyncRequest");    resetRequest(); return 1;
+            case kAsioLatenciesChanged:  JUCE_ASIO_LOG ("kAsioLatenciesChanged"); return 1;
+            case kAsioEngineVersion:     return 2;
 
             case kAsioSupportsTimeInfo:
             case kAsioSupportsTimeCode:  return 0;
-            case kAsioOverload: xruns++; return 1;
+            case kAsioOverload:          ++xruns; return 1;
         }
 
         return 0;
@@ -1382,29 +1373,30 @@ private:
     {
         static ASIOTime* JUCE_ASIOCALLBACK bufferSwitchTimeInfoCallback (ASIOTime*, long index, long)
         {
-            if (currentASIODev[deviceIndex] != nullptr)
-                currentASIODev[deviceIndex]->callback (index);
+            if (auto* d = currentASIODev[deviceIndex])
+                d->callback (index);
 
-            return nullptr;
+            return {};
         }
 
         static void JUCE_ASIOCALLBACK bufferSwitchCallback (long index, long)
         {
-            if (currentASIODev[deviceIndex] != nullptr)
-                currentASIODev[deviceIndex]->callback (index);
+            if (auto* d = currentASIODev[deviceIndex])
+                d->callback (index);
         }
 
         static long JUCE_ASIOCALLBACK asioMessagesCallback (long selector, long value, void*, double*)
         {
-            return currentASIODev[deviceIndex] != nullptr
-                     ? currentASIODev[deviceIndex]->asioMessagesCallback (selector, value)
-                     : 0;
+            if (auto* d = currentASIODev[deviceIndex])
+                return d->asioMessagesCallback (selector, value);
+
+            return {};
         }
 
         static void JUCE_ASIOCALLBACK sampleRateChangedCallback (ASIOSampleRate)
         {
-            if (currentASIODev[deviceIndex] != nullptr)
-                currentASIODev[deviceIndex]->resetRequest();
+            if (auto* d = currentASIODev[deviceIndex])
+                d->resetRequest();
         }
 
         static void setCallbacks (ASIOCallbacks& callbacks) noexcept
@@ -1426,18 +1418,14 @@ private:
 
     void setCallbackFunctions() noexcept
     {
-        /**/ if (currentASIODev[0] == this) ASIOCallbackFunctions<0>::setCallbacks (callbacks);
-        else if (currentASIODev[1] == this) ASIOCallbackFunctions<1>::setCallbacks (callbacks);
-        else if (currentASIODev[2] == this) ASIOCallbackFunctions<2>::setCallbacks (callbacks);
-        else if (currentASIODev[3] == this) ASIOCallbackFunctions<3>::setCallbacks (callbacks);
-        else jassertfalse;
+        ASIOCallbackFunctions<0>::setCallbacksForDevice (callbacks, this);
     }
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ASIOAudioIODevice)
 };
 
 template <>
-struct ASIOAudioIODevice::ASIOCallbackFunctions <sizeof(currentASIODev) / sizeof(currentASIODev[0])>
+struct ASIOAudioIODevice::ASIOCallbackFunctions<maxNumASIODevices>
 {
     static void setCallbacksForDevice (ASIOCallbacks&, ASIOAudioIODevice*) noexcept {}
 };
@@ -1449,10 +1437,9 @@ public:
     ASIOAudioIODeviceType() : AudioIODeviceType ("ASIO") {}
 
     //==============================================================================
-    void scanForDevices()
+    void scanForDevices() override
     {
         hasScanned = true;
-
         deviceNames.clear();
         classIds.clear();
 
@@ -1461,7 +1448,7 @@ public:
 
         if (RegOpenKey (HKEY_LOCAL_MACHINE, _T("software\\asio"), &hk) == ERROR_SUCCESS)
         {
-            TCHAR name [256];
+            TCHAR name[256] = {};
 
             while (RegEnumKey (hk, index++, name, numElementsInArray (name)) == ERROR_SUCCESS)
             {
@@ -1475,14 +1462,13 @@ public:
         }
     }
 
-    StringArray getDeviceNames (bool /*wantInputNames*/) const
+    StringArray getDeviceNames (bool /*wantInputNames*/) const override
     {
         jassert (hasScanned); // need to call scanForDevices() before doing this
-
         return deviceNames;
     }
 
-    int getDefaultDeviceIndex (bool) const
+    int getDefaultDeviceIndex (bool) const override
     {
         jassert (hasScanned); // need to call scanForDevices() before doing this
 
@@ -1501,8 +1487,8 @@ public:
 
     static int findFreeSlot()
     {
-        for (int i = 0; i < numElementsInArray (currentASIODev); ++i)
-            if (currentASIODev[i] == 0)
+        for (int i = 0; i < maxNumASIODevices; ++i)
+            if (currentASIODev[i] == nullptr)
                 return i;
 
         jassertfalse;  // unfortunately you can only have a finite number
@@ -1510,29 +1496,29 @@ public:
         return -1;
     }
 
-    int getIndexOfDevice (AudioIODevice* d, bool /*asInput*/) const
+    int getIndexOfDevice (AudioIODevice* d, bool /*asInput*/) const override
     {
         jassert (hasScanned); // need to call scanForDevices() before doing this
 
         return d == nullptr ? -1 : deviceNames.indexOf (d->getName());
     }
 
-    bool hasSeparateInputsAndOutputs() const    { return false; }
+    bool hasSeparateInputsAndOutputs() const override    { return false; }
 
     AudioIODevice* createDevice (const String& outputDeviceName,
-                                 const String& inputDeviceName)
+                                 const String& inputDeviceName) override
     {
         // ASIO can't open two different devices for input and output - they must be the same one.
         jassert (inputDeviceName == outputDeviceName || outputDeviceName.isEmpty() || inputDeviceName.isEmpty());
         jassert (hasScanned); // need to call scanForDevices() before doing this
 
-        const String deviceName (outputDeviceName.isNotEmpty() ? outputDeviceName
-                                                               : inputDeviceName);
-        const int index = deviceNames.indexOf (deviceName);
+        auto deviceName = outputDeviceName.isNotEmpty() ? outputDeviceName
+                                                        : inputDeviceName;
+        auto index = deviceNames.indexOf (deviceName);
 
         if (index >= 0)
         {
-            const int freeSlot = findFreeSlot();
+            auto freeSlot = findFreeSlot();
 
             if (freeSlot >= 0)
                 return new ASIOAudioIODevice (this, deviceName,
@@ -1564,7 +1550,7 @@ private:
         if (RegOpenKey (HKEY_CLASSES_ROOT, _T("clsid"), &hk) == ERROR_SUCCESS)
         {
             int index = 0;
-            TCHAR name [512];
+            TCHAR name[512] = {};
 
             while (RegEnumKey (hk, index++, name, numElementsInArray (name)) == ERROR_SUCCESS)
             {
@@ -1576,7 +1562,7 @@ private:
                     {
                         if (RegOpenKeyEx (subKey, _T("InprocServer32"), 0, KEY_READ, &pathKey) == ERROR_SUCCESS)
                         {
-                            TCHAR pathName [1024] = { 0 };
+                            TCHAR pathName[1024] = {};
                             DWORD dtype = REG_SZ;
                             DWORD dsize = sizeof (pathName);
 
@@ -1604,7 +1590,7 @@ private:
     //==============================================================================
     static bool isBlacklistedDriver (const String& driverName)
     {
-        return driverName == "ASIO DirectX Full Duplex Driver" || driverName == "ASIO Multimedia Driver";
+        return driverName.startsWith ("ASIO DirectX Full Duplex") || driverName == "ASIO Multimedia Driver";
     }
 
     void addDriverInfo (const String& keyName, HKEY hk)
@@ -1613,7 +1599,7 @@ private:
 
         if (RegOpenKeyEx (hk, keyName.toWideCharPointer(), 0, KEY_READ, &subKey) == ERROR_SUCCESS)
         {
-            TCHAR buf [256] = { 0 };
+            TCHAR buf[256] = {};
             DWORD dtype = REG_SZ;
             DWORD dsize = sizeof (buf);
 
@@ -1622,6 +1608,7 @@ private:
                 if (dsize > 0 && checkClassIsOk (buf))
                 {
                     CLSID classId;
+
                     if (CLSIDFromString ((LPOLESTR) buf, &classId) == S_OK)
                     {
                         dtype = REG_SZ;
