@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -28,14 +28,16 @@ namespace juce
     Wraps a pointer to a null-terminated UTF-16 character string, and provides
     various methods to operate on the data.
     @see CharPointer_UTF8, CharPointer_UTF32
+
+    @tags{Core}
 */
-class CharPointer_UTF16
+class CharPointer_UTF16  final
 {
 public:
    #if JUCE_NATIVE_WCHAR_IS_UTF16
-    typedef wchar_t CharType;
+    using CharType = wchar_t;
    #else
-    typedef int16 CharType;
+    using CharType = int16;
    #endif
 
     inline explicit CharPointer_UTF16 (const CharType* rawPointer) noexcept
@@ -43,10 +45,7 @@ public:
     {
     }
 
-    inline CharPointer_UTF16 (const CharPointer_UTF16& other) noexcept
-        : data (other.data)
-    {
-    }
+    inline CharPointer_UTF16 (const CharPointer_UTF16& other) = default;
 
     inline CharPointer_UTF16 operator= (CharPointer_UTF16 other) noexcept
     {
