@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -36,8 +35,10 @@ namespace juce
     horizontal scale factor.
 
     @see Typeface
+
+    @tags{Graphics}
 */
-class JUCE_API  Font
+class JUCE_API  Font  final
 {
 public:
     //==============================================================================
@@ -132,7 +133,7 @@ public:
 
         This may also be set to Font::getDefaultSansSerifFontName(), Font::getDefaultSerifFontName(),
         or Font::getDefaultMonospacedFontName(), which are not actual platform-specific font family names,
-        but are generic font familiy names that are used to represent the various default fonts.
+        but are generic font family names that are used to represent the various default fonts.
 
         If you need to know the exact typeface font family being used, you can call
         Font::getTypeface()->getName(), which will give you the platform-specific font family.
@@ -392,7 +393,7 @@ public:
         An extra x offset is added at the end of the run, to indicate where the right hand
         edge of the last character is.
     */
-    void getGlyphPositions (const String& text, Array <int>& glyphs, Array <float>& xOffsets) const;
+    void getGlyphPositions (const String& text, Array<int>& glyphs, Array<float>& xOffsets) const;
 
     //==============================================================================
     /** Returns the typeface used by this font.
