@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -37,6 +36,17 @@ static bool isUsingOldRotationMethod() noexcept
     return isPreV8;
 }
 
+static UIInterfaceOrientation getWindowOrientation()
+{
+    UIApplication* sharedApplication = [UIApplication sharedApplication];
+
+   #if (defined (__IPHONE_13_0) && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_13_0)
+    return [[[[sharedApplication windows] firstObject] windowScene] interfaceOrientation];
+   #else
+    return [sharedApplication statusBarOrientation];
+   #endif
+}
+
 namespace Orientations
 {
     static Desktop::DisplayOrientation convertToJuce (UIInterfaceOrientation orientation)
@@ -47,6 +57,7 @@ namespace Orientations
             case UIInterfaceOrientationPortraitUpsideDown:  return Desktop::upsideDown;
             case UIInterfaceOrientationLandscapeLeft:       return Desktop::rotatedClockwise;
             case UIInterfaceOrientationLandscapeRight:      return Desktop::rotatedAntiClockwise;
+            case UIInterfaceOrientationUnknown:
             default:                                        jassertfalse; // unknown orientation!
         }
 
@@ -61,6 +72,7 @@ namespace Orientations
             case Desktop::upsideDown:                       return UIInterfaceOrientationPortraitUpsideDown;
             case Desktop::rotatedClockwise:                 return UIInterfaceOrientationLandscapeLeft;
             case Desktop::rotatedAntiClockwise:             return UIInterfaceOrientationLandscapeRight;
+            case Desktop::allOrientations:
             default:                                        jassertfalse; // unknown orientation!
         }
 
@@ -76,7 +88,9 @@ namespace Orientations
                 case Desktop::upsideDown:             return CGAffineTransformMake (-1, 0,  0, -1, 0, 0);
                 case Desktop::rotatedClockwise:       return CGAffineTransformMake (0, -1,  1,  0, 0, 0);
                 case Desktop::rotatedAntiClockwise:   return CGAffineTransformMake (0,  1, -1,  0, 0, 0);
-                default: break;
+                case Desktop::upright:
+                case Desktop::allOrientations:
+                default:                              break;
             }
         }
 
@@ -97,6 +111,23 @@ namespace Orientations
     }
 }
 
+struct AsyncBoundsUpdater  : public AsyncUpdater
+{
+    AsyncBoundsUpdater (UIViewController* vc)
+        : viewController (vc)
+    {
+    }
+
+    ~AsyncBoundsUpdater() override
+    {
+        cancelPendingUpdate();
+    }
+
+    void handleAsyncUpdate() override;
+
+    UIViewController* viewController;
+};
+
 //==============================================================================
 } // namespace juce
 
@@ -130,8 +161,12 @@ using namespace juce;
 //==============================================================================
 @interface JuceUIViewController : UIViewController
 {
+@public
+    std::unique_ptr<AsyncBoundsUpdater> boundsUpdater;
 }
 
+- (JuceUIViewController*) init;
+
 - (NSUInteger) supportedInterfaceOrientations;
 - (BOOL) shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation) interfaceOrientation;
 - (void) willRotateToInterfaceOrientation: (UIInterfaceOrientation) toInterfaceOrientation duration: (NSTimeInterval) duration;
@@ -163,12 +198,21 @@ using namespace juce;
 namespace juce
 {
 
+struct UIViewPeerControllerReceiver
+{
+    virtual ~UIViewPeerControllerReceiver();
+    virtual void setViewController (UIViewController*) = 0;
+};
+
+UIViewPeerControllerReceiver::~UIViewPeerControllerReceiver() {}
+
 class UIViewComponentPeer  : public ComponentPeer,
-                             public FocusChangeListener
+                             public FocusChangeListener,
+                             public UIViewPeerControllerReceiver
 {
 public:
     UIViewComponentPeer (Component&, int windowStyleFlags, UIView* viewToAttachTo);
-    ~UIViewComponentPeer();
+    ~UIViewComponentPeer() override;
 
     //==============================================================================
     void* getNativeHandle() const override                  { return view; }
@@ -176,10 +220,18 @@ public:
     void setTitle (const String& title) override;
     void setBounds (const Rectangle<int>&, bool isNowFullScreen) override;
 
+    void setViewController (UIViewController* newController) override
+    {
+        jassert (controller == nullptr);
+        controller = [newController retain];
+    }
+
     Rectangle<int> getBounds() const override               { return getBounds (! isSharedWindow); }
     Rectangle<int> getBounds (bool global) const;
     Point<float> localToGlobal (Point<float> relativePosition) override;
     Point<float> globalToLocal (Point<float> screenPosition) override;
+    using ComponentPeer::localToGlobal;
+    using ComponentPeer::globalToLocal;
     void setAlpha (float newAlpha) override;
     void setMinimised (bool) override                       {}
     bool isMinimised() const override                       { return false; }
@@ -218,9 +270,8 @@ public:
     //==============================================================================
     UIWindow* window;
     JuceUIView* view;
-    JuceUIViewController* controller;
+    UIViewController* controller;
     bool isSharedWindow, fullScreen, insideDrawRect, isAppex;
-    static ModifierKeys currentModifiers;
 
     static int64 getMouseTime (UIEvent* e) noexcept
     {
@@ -234,7 +285,7 @@ public:
         {
             const Rectangle<int> screen (convertToRectInt ([UIScreen mainScreen].bounds));
 
-            switch ([[UIApplication sharedApplication] statusBarOrientation])
+            switch (getWindowOrientation())
             {
                 case UIInterfaceOrientationPortrait:
                     return r;
@@ -251,6 +302,7 @@ public:
                     return Rectangle<int> (screen.getWidth() - r.getBottom(), r.getX(),
                                            r.getHeight(), r.getWidth());
 
+                case UIInterfaceOrientationUnknown:
                 default: jassertfalse; // unknown orientation!
             }
         }
@@ -264,7 +316,7 @@ public:
         {
             const Rectangle<int> screen (convertToRectInt ([UIScreen mainScreen].bounds));
 
-            switch ([[UIApplication sharedApplication] statusBarOrientation])
+            switch (getWindowOrientation())
             {
                 case UIInterfaceOrientationPortrait:
                     return r;
@@ -281,6 +333,7 @@ public:
                     return Rectangle<int> (r.getY(), screen.getWidth() - r.getRight(),
                                            r.getHeight(), r.getWidth());
 
+                case UIInterfaceOrientationUnknown:
                 default: jassertfalse; // unknown orientation!
             }
         }
@@ -320,6 +373,11 @@ static void sendScreenBoundsUpdate (JuceUIViewController* c)
         juceView->owner->updateTransformAndScreenBounds();
 }
 
+void AsyncBoundsUpdater::handleAsyncUpdate()
+{
+    sendScreenBoundsUpdate ((JuceUIViewController*) viewController);
+}
+
 static bool isKioskModeView (JuceUIViewController* c)
 {
     JuceUIView* juceView = (JuceUIView*) [c view];
@@ -336,6 +394,15 @@ MultiTouchMapper<UITouch*> UIViewComponentPeer::currentTouches;
 //==============================================================================
 @implementation JuceUIViewController
 
+- (JuceUIViewController*) init
+{
+    self = [super init];
+
+    boundsUpdater = std::make_unique<AsyncBoundsUpdater> (self);
+
+    return self;
+}
+
 - (NSUInteger) supportedInterfaceOrientations
 {
     return Orientations::getSupportedOrientations();
@@ -368,7 +435,8 @@ MultiTouchMapper<UITouch*> UIViewComponentPeer::currentTouches;
 
     // On some devices the screen-size isn't yet updated at this point, so also trigger another
     // async update to double-check..
-    MessageManager::callAsync ([=]() { sendScreenBoundsUpdate (self); });
+    if (boundsUpdater != nullptr)
+        boundsUpdater->triggerAsyncUpdate();
 }
 
 - (BOOL) prefersStatusBarHidden
@@ -376,6 +444,13 @@ MultiTouchMapper<UITouch*> UIViewComponentPeer::currentTouches;
     return isKioskModeView (self);
 }
 
+#if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
+ - (BOOL) prefersHomeIndicatorAutoHidden
+ {
+     return isKioskModeView (self);
+ }
+#endif
+
 - (UIStatusBarStyle) preferredStatusBarStyle
 {
     return UIStatusBarStyleDefault;
@@ -536,18 +611,6 @@ bool KeyPress::isKeyCurrentlyDown (int)
     return false;
 }
 
-ModifierKeys UIViewComponentPeer::currentModifiers;
-
-ModifierKeys ModifierKeys::getCurrentModifiersRealtime() noexcept
-{
-    return UIViewComponentPeer::currentModifiers;
-}
-
-void ModifierKeys::updateCurrentModifiers() noexcept
-{
-    currentModifiers = UIViewComponentPeer::currentModifiers;
-}
-
 Point<float> juce_lastMousePos;
 
 //==============================================================================
@@ -589,7 +652,6 @@ UIViewComponentPeer::UIViewComponentPeer (Component& comp, const int windowStyle
         window.rootViewController = controller;
 
         window.hidden = true;
-        window.autoresizesSubviews = NO;
         window.transform = Orientations::getCGTransformFor (Desktop::getInstance().getCurrentOrientation());
         window.opaque = component.isOpaque();
         window.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent: 0];
@@ -610,8 +672,11 @@ UIViewComponentPeer::UIViewComponentPeer (Component& comp, const int windowStyle
 
 UIViewComponentPeer::~UIViewComponentPeer()
 {
+    currentTouches.deleteAllTouchesForPeer (this);
     Desktop::getInstance().removeFocusChangeListener (this);
 
+    ((JuceUIViewController*) controller)->boundsUpdater = nullptr;
+
     view->owner = nullptr;
     [view removeFromSuperview];
     [view release];
@@ -714,7 +779,7 @@ void UIViewComponentPeer::updateTransformAndScreenBounds()
     const Rectangle<int> oldArea (component.getBounds());
     const Rectangle<int> oldDesktop (desktop.getDisplays().getMainDisplay().userArea);
 
-    const_cast<Desktop::Displays&> (desktop.getDisplays()).refresh();
+    const_cast<Displays&> (desktop.getDisplays()).refresh();
 
     window.transform = Orientations::getCGTransformFor (desktop.getCurrentOrientation());
     view.transform = CGAffineTransformIdentity;
@@ -841,17 +906,17 @@ void UIViewComponentPeer::handleTouches (UIEvent* event, const bool isDown, cons
         juce_lastMousePos = pos + getBounds (true).getPosition().toFloat();
 
         const int64 time = getMouseTime (event);
-        const int touchIndex = currentTouches.getIndexOfTouch (touch);
+        const int touchIndex = currentTouches.getIndexOfTouch (this, touch);
 
-        ModifierKeys modsToSend (currentModifiers);
+        ModifierKeys modsToSend (ModifierKeys::currentModifiers);
 
         if (isDown)
         {
             if ([touch phase] != UITouchPhaseBegan)
                 continue;
 
-            currentModifiers = currentModifiers.withoutMouseButtons().withFlags (ModifierKeys::leftButtonModifier);
-            modsToSend = currentModifiers;
+            ModifierKeys::currentModifiers = ModifierKeys::currentModifiers.withoutMouseButtons().withFlags (ModifierKeys::leftButtonModifier);
+            modsToSend = ModifierKeys::currentModifiers;
 
             // this forces a mouse-enter/up event, in case for some reason we didn't get a mouse-up before.
             handleMouseEvent (MouseInputSource::InputSourceType::touch, pos, modsToSend.withoutMouseButtons(),
@@ -875,7 +940,7 @@ void UIViewComponentPeer::handleTouches (UIEvent* event, const bool isDown, cons
         if (isCancel)
         {
             currentTouches.clearTouch (touchIndex);
-            modsToSend = currentModifiers = currentModifiers.withoutMouseButtons();
+            modsToSend = ModifierKeys::currentModifiers = ModifierKeys::currentModifiers.withoutMouseButtons();
         }
 
         // NB: some devices return 0 or 1.0 if pressure is unknown, so we'll clip our value to a believable range:
@@ -890,7 +955,7 @@ void UIViewComponentPeer::handleTouches (UIEvent* event, const bool isDown, cons
 
         if (isUp || isCancel)
         {
-            handleMouseEvent (MouseInputSource::InputSourceType::touch, Point<float> (-1.0f, -1.0f), modsToSend,
+            handleMouseEvent (MouseInputSource::InputSourceType::touch, MouseInputSource::offscreenMousePos, modsToSend,
                               MouseInputSource::invalidPressure, MouseInputSource::invalidOrientation, time, {}, touchIndex);
 
             if (! isValidPeer (this))
@@ -1029,7 +1094,7 @@ void UIViewComponentPeer::drawRect (CGRect r)
 
     // NB the CTM on iOS already includes a factor for the display scale, so
     // we'll tell the context that the scale is 1.0 to avoid it using it twice
-    CoreGraphicsContext g (cg, getComponent().getHeight(), 1.0f);
+    CoreGraphicsContext g (cg, getComponent().getHeight());
 
     insideDrawRect = true;
     handlePaint (g);
@@ -1074,7 +1139,7 @@ void Desktop::allowedOrientationsChanged()
         jassert (i < n);
         i = jmin (n - 1, i);
 
-        NSNumber *value = [NSNumber numberWithInt:Orientations::convertFromJuce (orientations[i])];
+        NSNumber *value = [NSNumber numberWithInt: (int) Orientations::convertFromJuce (orientations[i])];
         [[UIDevice currentDevice] setValue:value forKey:@"orientation"];
         [value release];
     }
