@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -38,6 +37,8 @@ namespace juce
     User-code should very rarely need to have any involvement with this class.
 
     @see Component::createNewPeer
+
+    @tags{GUI}
 */
 class JUCE_API  ComponentPeer
 {
@@ -70,7 +71,7 @@ public:
         windowIgnoresKeyPresses     = (1 << 10),   /**< Tells the window not to catch any keypresses. This can
                                                         be used for things like plugin windows, to stop them interfering
                                                         with the host's shortcut keys */
-        windowIsSemiTransparent     = (1 << 31)    /**< Not intended for public use - makes a window transparent. */
+        windowIsSemiTransparent     = (1 << 30)    /**< Not intended for public use - makes a window transparent. */
 
     };
 
@@ -313,7 +314,7 @@ public:
 
     //==============================================================================
     void handleMouseEvent (MouseInputSource::InputSourceType type, Point<float> positionWithinPeer, ModifierKeys newMods, float pressure,
-                           float orientation, int64 time, PenDetails pen = { 0.0f, 0.0f, 0.0f }, int touchIndex = 0);
+                           float orientation, int64 time, PenDetails pen = PenDetails(), int touchIndex = 0);
 
     void handleMouseWheel (MouseInputSource::InputSourceType type, Point<float> positionWithinPeer,
                            int64 time, const MouseWheelDetails&, int touchIndex = 0);
@@ -323,6 +324,7 @@ public:
 
     void handleUserClosingWindow();
 
+    /** Structure to describe drag and drop information */
     struct DragInfo
     {
         StringArray files;
@@ -361,12 +363,57 @@ public:
     virtual int getCurrentRenderingEngine() const;
     virtual void setCurrentRenderingEngine (int index);
 
+    //==============================================================================
+    /** On desktop platforms this method will check all the mouse and key states and return
+        a ModifierKeys object representing them.
+
+        This isn't recommended and is only needed in special circumstances for up-to-date
+        modifier information at times when the app's event loop isn't running normally.
+
+        Another reason to avoid this method is that it's not stateless and calling it may
+        update the ModifierKeys::currentModifiers object, which could cause subtle changes
+        in the behaviour of some components.
+    */
+    static ModifierKeys getCurrentModifiersRealtime() noexcept;
+
+    //==============================================================================
+    /**  Used to receive callbacks when the OS scale factor of this ComponentPeer changes.
+
+         This is used internally by some native JUCE windows on Windows and Linux and you
+         shouldn't need to worry about it in your own code unless you are dealing directly
+         with native windows.
+    */
+    struct JUCE_API  ScaleFactorListener
+    {
+        /** Destructor. */
+        virtual ~ScaleFactorListener() = default;
+
+        /** Called when the scale factor changes. */
+        virtual void nativeScaleFactorChanged (double newScaleFactor) = 0;
+    };
+
+    /** Adds a scale factor listener. */
+    void addScaleFactorListener (ScaleFactorListener* listenerToAdd)          { scaleFactorListeners.add (listenerToAdd); }
+
+    /** Removes a scale factor listener. */
+    void removeScaleFactorListener (ScaleFactorListener* listenerToRemove)    { scaleFactorListeners.remove (listenerToRemove);  }
+
+    //==============================================================================
+    /** On Windows and Linux this will return the OS scaling factor currently being applied
+        to the native window. This is used to convert between physical and logical pixels
+        at the OS API level and you shouldn't need to use it in your own code unless you
+        are dealing directly with the native window.
+    */
+    virtual double getPlatformScaleFactor() const noexcept    { return 1.0; }
+
 protected:
     //==============================================================================
     Component& component;
     const int styleFlags;
     Rectangle<int> lastNonFullscreenBounds;
     ComponentBoundsConstrainer* constrainer = nullptr;
+    static std::function<ModifierKeys()> getNativeRealtimeModifiers;
+    ListenerList<ScaleFactorListener> scaleFactorListeners;
 
 private:
     //==============================================================================
