@@ -32,8 +32,6 @@
 #include "../misc/Time.h"
 #include "../text/StringArray.h"
 
-#include "CarlaJuceUtils.hpp"
-
 #ifdef CARLA_OS_WIN
 # include <shlobj.h>
 #else
@@ -54,6 +52,11 @@
 
 namespace water {
 
+File::File () noexcept
+    : fullPath ()
+{
+}
+
 File::File (const String& fullPathName)
     : fullPath (parseAbsolutePath (fullPathName))
 {
@@ -96,6 +99,16 @@ File& File::operator= (File&& other) noexcept
 }
 #endif
 
+bool File::isNull() const
+{
+    return fullPath.isEmpty();
+}
+
+bool File::isNotNull() const
+{
+    return fullPath.isNotEmpty();
+}
+
 //==============================================================================
 static String removeEllipsis (const String& path)
 {
@@ -189,7 +202,7 @@ String File::parseAbsolutePath (const String& p)
     // Yes, I know it's legal for a unix pathname to contain a backslash, but this assertion is here
     // to catch anyone who's trying to run code that was written on Windows with hard-coded path names.
     // If that's why you've ended up here, use File::getChildFile() to build your paths instead.
-    jassert ((! p.containsChar ('\\')) || (p.indexOfChar ('/') >= 0 && p.indexOfChar ('/') < p.indexOfChar ('\\')));
+    wassert ((! p.containsChar ('\\')) || (p.indexOfChar ('/') >= 0 && p.indexOfChar ('/') < p.indexOfChar ('\\')));
 
     String path (removeEllipsis (p));
 
@@ -212,15 +225,6 @@ String File::parseAbsolutePath (const String& p)
     }
     else if (! path.startsWithChar (separator))
     {
-        /*  When you supply a raw string to the File object constructor, it must be an absolute path.
-            If you're trying to parse a string that may be either a relative path or an absolute path,
-            you MUST provide a context against which the partial path can be evaluated - you can do
-            this by simply using File::getChildFile() instead of the File constructor. E.g. saying
-            "File::getCurrentWorkingDirectory().getChildFile (myUnknownPath)" would return an absolute
-            path if that's what was supplied, or would evaluate a partial path relative to the CWD.
-        */
-        CARLA_SAFE_ASSERT_RETURN(path.startsWith ("./") || path.startsWith ("../"), String());
-
         return File::getCurrentWorkingDirectory().getChildFile (path).getFullPathName();
     }
 #endif
@@ -328,8 +332,21 @@ bool File::copyDirectoryTo (const File& newDirectory) const
         findChildFiles (subFiles, File::findFiles, false);
 
         for (int i = 0; i < subFiles.size(); ++i)
-            if (! subFiles.getReference(i).copyFileTo (newDirectory.getChildFile (subFiles.getReference(i).getFileName())))
-                return false;
+        {
+            const File& src (subFiles.getReference(i));
+            const File& dst (newDirectory.getChildFile (src.getFileName()));
+
+            if (src.isSymbolicLink())
+            {
+                if (! src.getLinkedTarget().createSymbolicLink (dst, true))
+                    return false;
+            }
+            else
+            {
+                if (! src.copyFileTo (dst))
+                    return false;
+            }
+        }
 
         subFiles.clear();
         findChildFiles (subFiles, File::findDirectories, false);
@@ -706,7 +723,7 @@ File File::withFileExtension (StringRef newExtension) const
 //==============================================================================
 FileInputStream* File::createInputStream() const
 {
-    ScopedPointer<FileInputStream> fin (new FileInputStream (*this));
+    CarlaScopedPointer<FileInputStream> fin (new FileInputStream (*this));
 
     if (fin->openedOk())
         return fin.release();
@@ -716,7 +733,7 @@ FileInputStream* File::createInputStream() const
 
 FileOutputStream* File::createOutputStream (const size_t bufferSize) const
 {
-    ScopedPointer<FileOutputStream> out (new FileOutputStream (*this, bufferSize));
+    CarlaScopedPointer<FileOutputStream> out (new FileOutputStream (*this, bufferSize));
 
     return out->failedToOpen() ? nullptr
                                : out.release();
@@ -726,7 +743,7 @@ FileOutputStream* File::createOutputStream (const size_t bufferSize) const
 bool File::appendData (const void* const dataToAppend,
                        const size_t numberOfBytes) const
 {
-    jassert (((ssize_t) numberOfBytes) >= 0);
+    wassert (((ssize_t) numberOfBytes) >= 0);
 
     if (numberOfBytes == 0)
         return true;
@@ -973,7 +990,9 @@ namespace WindowsFileHelpers
 
     int64 fileTimeToTime (const FILETIME* const ft)
     {
-        static_jassert (sizeof (ULARGE_INTEGER) == sizeof (FILETIME)); // tell me if this fails!
+#ifdef CARLA_PROPER_CPP11_SUPPORT
+        static_wassert (sizeof (ULARGE_INTEGER) == sizeof (FILETIME)); // tell me if this fails!
+#endif
 
         return (int64) ((reinterpret_cast<const ULARGE_INTEGER*> (ft)->QuadPart - 116444736000000000LL) / 10000);
     }
@@ -1052,6 +1071,11 @@ bool File::deleteFile() const
                          : DeleteFile (fullPath.toUTF8()) != 0;
 }
 
+File File::getLinkedTarget() const
+{
+    return *this;
+}
+
 bool File::copyInternal (const File& dest) const
 {
     return CopyFile (fullPath.toUTF8(), dest.getFullPathName().toUTF8(), false) != 0;
@@ -1115,7 +1139,7 @@ File File::getSpecialLocation (const SpecialLocationType type)
             return WindowsFileHelpers::getModuleFileName (nullptr);
 
         default:
-            jassertfalse; // unknown type?
+            wassertfalse; // unknown type?
             return File();
     }
 
@@ -1353,7 +1377,7 @@ File water_getExecutableFile()
             }
 
             // if we reach this, we failed to find ourselves...
-            jassertfalse;
+            wassertfalse;
             return filename;
         }
     };
@@ -1432,7 +1456,7 @@ File File::getSpecialLocation (const SpecialLocationType type)
         }
 
         default:
-            jassertfalse; // unknown type?
+            wassertfalse; // unknown type?
             break;
     }
 
@@ -1583,7 +1607,7 @@ File File::getSpecialLocation (const SpecialLocationType type)
         }
 
         default:
-            jassertfalse; // unknown type?
+            wassertfalse; // unknown type?
             break;
     }
 
