@@ -54,13 +54,13 @@ public:
     {
     }
 
-    inline CharPointer_UTF8 operator= (CharPointer_UTF8 other) noexcept
+    inline CharPointer_UTF8& operator= (CharPointer_UTF8 other) noexcept
     {
         data = other.data;
         return *this;
     }
 
-    inline CharPointer_UTF8 operator= (const CharType* text) noexcept
+    inline CharPointer_UTF8& operator= (const CharType* text) noexcept
     {
         data = const_cast<CharType*> (text);
         return *this;
@@ -122,14 +122,14 @@ public:
     /** Moves this pointer along to the next character in the string. */
     CharPointer_UTF8& operator++() noexcept
     {
-        jassert (*data != 0); // trying to advance past the end of the string?
+        wassert (*data != 0); // trying to advance past the end of the string?
         const signed char n = (signed char) *data++;
 
         if (n < 0)
         {
             water_uchar bit = 0x40;
 
-            while ((n & bit) != 0 && bit > 0x8)
+            while ((static_cast<unsigned char>(n) & bit) != 0 && bit > 0x8)
             {
                 ++data;
                 bit >>= 1;
@@ -140,7 +140,7 @@ public:
     }
 
     /** Moves this pointer back to the previous character in the string. */
-    CharPointer_UTF8 operator--() noexcept
+    CharPointer_UTF8& operator--() noexcept
     {
         int count = 0;
 
@@ -282,7 +282,7 @@ public:
     */
     size_t sizeInBytes() const noexcept
     {
-        jassert (data != nullptr);
+        wassert (data != nullptr);
         return strlen (data) + 1;
     }
 
@@ -556,7 +556,7 @@ public:
     */
     static bool isByteOrderMark (const void* possibleByteOrder) noexcept
     {
-        jassert (possibleByteOrder != nullptr);
+        wassert (possibleByteOrder != nullptr);
         const uint8* const c = static_cast<const uint8*> (possibleByteOrder);
 
         return c[0] == (uint8) byteOrderMark1
