@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2013-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,24 +23,10 @@
 #include "CarlaPipeUtils.hpp"
 #include "CarlaString.hpp"
 
-#ifdef USING_JUCE
-# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wconversion"
-#  pragma GCC diagnostic ignored "-Weffc++"
-#  pragma GCC diagnostic ignored "-Wsign-conversion"
-#  pragma GCC diagnostic ignored "-Wundef"
-#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
-# endif
-# include "AppConfig.h"
-# include "juce_events/juce_events.h"
-# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-#  pragma GCC diagnostic pop
-# endif
-#endif
-
 #include "water/files/File.h"
 
+static const char* const kPathForCarlaFiles = "carlafiles";
+
 template<>
 void Lv2PluginBaseClass<NativeTimeInfo>::clearTimeData() noexcept
 {
@@ -69,9 +55,7 @@ public:
 #endif
           kIgnoreParameters(std::strncmp(desc->label, "carla", 5) == 0),
           fMidiEventCount(0),
-#ifdef USING_JUCE
-          fJuceInitialiser(),
-#endif
+          fLastProjectPath(),
           fLoadedFile(),
           fWorkerUISignal(0)
     {
@@ -276,7 +260,7 @@ public:
 
                         if (obj->body.otype == fURIs.patchSet) {
                             // Get property URI.
-                            const LV2_Atom* property = NULL;
+                            const LV2_Atom* property = nullptr;
                             lv2_atom_object_get(obj, fURIs.patchProperty, &property, 0);
                             CARLA_SAFE_ASSERT_CONTINUE(property != nullptr);
                             CARLA_SAFE_ASSERT_CONTINUE(property->type == fURIs.atomURID);
@@ -292,7 +276,7 @@ public:
                             }
 
                             // Get value.
-                            const LV2_Atom* fileobj = NULL;
+                            const LV2_Atom* fileobj = nullptr;
                             lv2_atom_object_get(obj, fURIs.patchValue, &fileobj, 0);
                             CARLA_SAFE_ASSERT_CONTINUE(fileobj != nullptr);
                             CARLA_SAFE_ASSERT_CONTINUE(fileobj->type == fURIs.atomPath);
@@ -410,9 +394,54 @@ public:
 
     // ----------------------------------------------------------------------------------------------------------------
 
-    LV2_State_Status lv2_save(const LV2_State_Store_Function store, const LV2_State_Handle handle,
-                              const uint32_t /*flags*/, const LV2_Feature* const* const /*features*/) const
+    void cleanupLastProjectPath()
     {
+        fLastProjectPath.clear();
+    }
+
+    void saveLastProjectPathIfPossible(const LV2_Feature* const* const features)
+    {
+        if (features == nullptr)
+            return cleanupLastProjectPath();
+
+        const LV2_State_Free_Path* freePath = nullptr;
+        const LV2_State_Make_Path* makePath = nullptr;
+
+        for (int i=0; features[i] != nullptr; ++i)
+        {
+            /**/ if (freePath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__freePath) == 0)
+                freePath = (const LV2_State_Free_Path*)features[i]->data;
+            else if (makePath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__makePath) == 0)
+                makePath = (const LV2_State_Make_Path*)features[i]->data;
+        }
+
+        if (makePath == nullptr || makePath->path == nullptr)
+            return cleanupLastProjectPath();
+
+        if (freePath == nullptr)
+            freePath = fFreePath;
+
+        char* const newpath = makePath->path(makePath->handle, kPathForCarlaFiles);
+
+        if (newpath == nullptr)
+            return cleanupLastProjectPath();
+
+        fLastProjectPath = CarlaString(water::File(newpath).getParentDirectory().getFullPathName().toRawUTF8());
+
+        if (freePath != nullptr && freePath->free_path != nullptr)
+            freePath->free_path(freePath->handle, newpath);
+#ifndef CARLA_OS_WIN
+        // this is not safe to call under windows
+        else
+            std::free(newpath);
+#endif
+    }
+
+    LV2_State_Status lv2_save(const LV2_State_Store_Function store, const LV2_State_Handle handle,
+                              const uint32_t /*flags*/, const LV2_Feature* const* const features)
+    {
+        saveLastProjectPathIfPossible(features);
+
         if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
         {
             store(handle,
@@ -429,7 +458,8 @@ public:
 
         if (char* const state = fDescriptor->get_state(fHandle))
         {
-            store(handle, fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"), state, std::strlen(state)+1, fURIs.atomString, LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
+            store(handle, fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"),
+                  state, std::strlen(state)+1, fURIs.atomString, LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
             std::free(state);
             return LV2_STATE_SUCCESS;
         }
@@ -438,8 +468,10 @@ public:
     }
 
     LV2_State_Status lv2_restore(const LV2_State_Retrieve_Function retrieve, const LV2_State_Handle handle,
-                                 uint32_t flags, const LV2_Feature* const* const /*features*/)
+                                 uint32_t flags, const LV2_Feature* const* const features)
     {
+        saveLastProjectPathIfPossible(features);
+
         size_t   size = 0;
         uint32_t type = 0;
 
@@ -573,12 +605,10 @@ public:
                 std::strcmp(features[i]->URI, LV2_EXTERNAL_UI_DEPRECATED_URI) == 0)
             {
                 fUI.host = (const LV2_External_UI_Host*)features[i]->data;
-                break;
             }
             if (std::strcmp(features[i]->URI, LV2_UI__touch) == 0)
             {
                 fUI.touch = (const LV2UI_Touch*)features[i]->data;
-                break;
             }
         }
 
@@ -745,6 +775,11 @@ protected:
             fUI.touch->touch(fUI.touch->handle, index+fPorts.indexOffset, touch);
     }
 
+    void handleUiResize(const uint32_t, const uint32_t) const
+    {
+        // nothing here
+    }
+
     void handleUiCustomDataChanged(const char* const key, const char* const value) const
     {
         carla_stdout("TODO: handleUiCustomDataChanged %s %s", key, value);
@@ -784,8 +819,13 @@ protected:
 
     intptr_t handleDispatcher(const NativeHostDispatcherOpcode opcode, const int32_t index, const intptr_t value, void* const ptr, const float opt)
     {
-        carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)",
-                    opcode, index, value, ptr, static_cast<double>(opt));
+#ifdef DEBUG
+        if (opcode != NATIVE_HOST_OPCODE_GET_FILE_PATH)
+        {
+            carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)",
+                        opcode, index, value, ptr, static_cast<double>(opt));
+        }
+#endif
 
         intptr_t ret = 0;
 
@@ -801,10 +841,15 @@ protected:
         case NATIVE_HOST_OPCODE_INTERNAL_PLUGIN:
         case NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY:
         case NATIVE_HOST_OPCODE_REQUEST_IDLE:
-        case NATIVE_HOST_OPCODE_GET_FILE_PATH:
             // nothing
             break;
 
+        case NATIVE_HOST_OPCODE_GET_FILE_PATH:
+            CARLA_SAFE_ASSERT_RETURN(ptr != nullptr, 0);
+            if (std::strcmp((char*)ptr, "carla") == 0 && fLastProjectPath != nullptr)
+                return static_cast<intptr_t>((uintptr_t)fLastProjectPath.buffer());
+            break;
+
         case NATIVE_HOST_OPCODE_UI_UNAVAILABLE:
             handleUiClosed();
             break;
@@ -813,6 +858,12 @@ protected:
             CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
             handleUiParameterTouch(static_cast<uint32_t>(index), value != 0);
             break;
+
+        case NATIVE_HOST_OPCODE_UI_RESIZE:
+            CARLA_SAFE_ASSERT_RETURN(index > 0, 0);
+            CARLA_SAFE_ASSERT_RETURN(value > 0, 0);
+            handleUiResize(static_cast<uint32_t>(index), static_cast<uint32_t>(value));
+            break;
         }
 
         return ret;
@@ -855,11 +906,9 @@ private:
     uint32_t        fMidiEventCount;
     NativeMidiEvent fMidiEvents[kMaxMidiEvents];
 
-#ifdef USING_JUCE
-    juce::SharedResourcePointer<juce::ScopedJuceInitialiser_GUI> fJuceInitialiser;
-#endif
-
+    CarlaString fLastProjectPath;
     CarlaString fLoadedFile;
+
     int fWorkerUISignal;
 
     // -------------------------------------------------------------------
@@ -1122,7 +1171,7 @@ static LV2UI_Handle lv2ui_instantiate(const LV2UI_Descriptor*, const char*, cons
 
 static void lv2ui_port_event(LV2UI_Handle ui, uint32_t portIndex, uint32_t bufferSize, uint32_t format, const void* buffer)
 {
-    carla_debug("lv2ui_port_eventxx(%p, %i, %i, %i, %p)", ui, portIndex, bufferSize, format, buffer);
+    carla_debug("lv2ui_port_event(%p, %i, %i, %i, %p)", ui, portIndex, bufferSize, format, buffer);
     uiPtr->lv2ui_port_event(portIndex, bufferSize, format, buffer);
 }
 
