@@ -42,22 +42,6 @@
 #include "CarlaMathUtils.hpp"
 #include "CarlaVstUtils.hpp"
 
-#ifdef USING_JUCE
-# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wconversion"
-#  pragma GCC diagnostic ignored "-Weffc++"
-#  pragma GCC diagnostic ignored "-Wsign-conversion"
-#  pragma GCC diagnostic ignored "-Wundef"
-#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
-# endif
-# include "AppConfig.h"
-# include "juce_events/juce_events.h"
-# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-#  pragma GCC diagnostic pop
-# endif
-#endif
-
 static uint32_t d_lastBufferSize = 0;
 static double   d_lastSampleRate = 0.0;
 
@@ -98,9 +82,6 @@ public:
           fUiLauncher(nullptr),
           fHostType(kHostTypeNull),
           fMidiOutEvents(),
-#ifdef USING_JUCE
-          fJuceInitialiser(),
-#endif
           fStateChunk(nullptr)
     {
         fHost.handle      = this;
@@ -150,15 +131,15 @@ public:
         fVstRect.top  = 0;
         fVstRect.left = 0;
 
-        if (kIsUsingUILauncher)
+        if (kIsUsingUILauncher || (fDescriptor->hints & NATIVE_PLUGIN_USES_UI_SIZE) == 0x0)
         {
-            fVstRect.bottom = ui_launcher_res::carla_uiHeight;
             fVstRect.right  = ui_launcher_res::carla_uiWidth;
+            fVstRect.bottom = ui_launcher_res::carla_uiHeight;
         }
         else
         {
-            fVstRect.bottom = 712;
-            fVstRect.right  = 1024;
+            fVstRect.right  = static_cast<int16_t>(fDescriptor->ui_width);
+            fVstRect.bottom = static_cast<int16_t>(fDescriptor->ui_height);
         }
 
         init();
@@ -639,7 +620,7 @@ public:
 
         if (const VstTimeInfo* const vstTimeInfo = (const VstTimeInfo*)hostCallback(audioMasterGetTime, 0, kWantVstTimeFlags))
         {
-            fTimeInfo.frame     = static_cast<uint64_t>(vstTimeInfo->samplePos);
+            fTimeInfo.frame     = static_cast<uint64_t>(std::max(0.0, vstTimeInfo->samplePos));
             fTimeInfo.playing   =  (vstTimeInfo->flags & kVstTransportPlaying);
             fTimeInfo.bbt.valid = ((vstTimeInfo->flags & kVstTempoValid) != 0 || (vstTimeInfo->flags & kVstTimeSigValid) != 0);
 
@@ -651,7 +632,7 @@ public:
             else
                 fTimeInfo.bbt.beatsPerMinute = 120.0;
 
-            if (vstTimeInfo->flags & (kVstPpqPosValid|kVstTimeSigValid))
+            if ((vstTimeInfo->flags & kVstPpqPosValid) != 0 && (vstTimeInfo->flags & kVstTimeSigValid) != 0)
             {
                 const double ppqPos    = std::abs(vstTimeInfo->ppqPos);
                 const int    ppqPerBar = vstTimeInfo->timeSigNumerator * 4 / vstTimeInfo->timeSigDenominator;
@@ -660,26 +641,38 @@ public:
 
                 fTimeInfo.bbt.bar         = static_cast<int32_t>(ppqPos) / ppqPerBar + 1;
                 fTimeInfo.bbt.beat        = static_cast<int32_t>(barBeats - rest + 0.5) + 1;
-                fTimeInfo.bbt.tick        = static_cast<int32_t>(rest * fTimeInfo.bbt.ticksPerBeat + 0.5);
+                fTimeInfo.bbt.tick        = rest * fTimeInfo.bbt.ticksPerBeat;
                 fTimeInfo.bbt.beatsPerBar = static_cast<float>(vstTimeInfo->timeSigNumerator);
                 fTimeInfo.bbt.beatType    = static_cast<float>(vstTimeInfo->timeSigDenominator);
 
                 if (vstTimeInfo->ppqPos < 0.0)
                 {
-                    --fTimeInfo.bbt.bar;
-                    fTimeInfo.bbt.beat = vstTimeInfo->timeSigNumerator - fTimeInfo.bbt.beat + 1;
-                    fTimeInfo.bbt.tick = fTimeInfo.bbt.ticksPerBeat - fTimeInfo.bbt.tick - 1;
+                    fTimeInfo.bbt.bar  = std::max(1, fTimeInfo.bbt.bar-1);
+                    fTimeInfo.bbt.beat = std::max(1, vstTimeInfo->timeSigNumerator - fTimeInfo.bbt.beat + 1);
+                    fTimeInfo.bbt.tick = std::max(0.0, fTimeInfo.bbt.ticksPerBeat - fTimeInfo.bbt.tick - 1);
                 }
             }
             else
             {
                 fTimeInfo.bbt.bar         = 1;
                 fTimeInfo.bbt.beat        = 1;
-                fTimeInfo.bbt.tick        = 0;
+                fTimeInfo.bbt.tick        = 0.0;
                 fTimeInfo.bbt.beatsPerBar = 4.0f;
                 fTimeInfo.bbt.beatType    = 4.0f;
             }
 
+#if 0
+            // for testing bad host values
+            static double last = -99999.0;
+            if (carla_isNotEqual(vstTimeInfo->samplePos, last))
+            {
+                last = vstTimeInfo->samplePos;
+                carla_stdout("time info %f %f %lu | %i %i",
+                             vstTimeInfo->samplePos, vstTimeInfo->ppqPos, fTimeInfo.frame,
+                             vstTimeInfo->timeSigNumerator, vstTimeInfo->timeSigDenominator);
+            }
+#endif
+
             fTimeInfo.bbt.barStartTick = fTimeInfo.bbt.ticksPerBeat *
                                          static_cast<double>(fTimeInfo.bbt.beatsPerBar) *
                                          (fTimeInfo.bbt.bar - 1);
@@ -745,6 +738,15 @@ protected:
         hostCallback(touch ? audioMasterBeginEdit : audioMasterEndEdit, static_cast<int32_t>(index));
     }
 
+    void handleUiResize(const int16_t width, const int16_t height)
+    {
+        if (kIsUsingUILauncher)
+            return;
+        fVstRect.right = width;
+        fVstRect.bottom = height;
+        hostCallback(audioMasterSizeWindow, width, height);
+    }
+
     void handleUiCustomDataChanged(const char* const /*key*/, const char* const /*value*/) const
     {
     }
@@ -797,6 +799,12 @@ protected:
             CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
             handleUiParameterTouch(static_cast<uint32_t>(index), value != 0);
             break;
+
+        case NATIVE_HOST_OPCODE_UI_RESIZE:
+            CARLA_SAFE_ASSERT_RETURN(index > 0 && index < INT16_MAX, 0);
+            CARLA_SAFE_ASSERT_RETURN(value > 0 && value < INT16_MAX, 0);
+            handleUiResize(static_cast<int16_t>(index), static_cast<int16_t>(value));
+            break;
         }
 
         // unused for now
@@ -865,10 +873,6 @@ private:
         CARLA_DECLARE_NON_COPY_STRUCT(FixedVstEvents);
     } fMidiOutEvents;
 
-#ifdef USING_JUCE
-    juce::SharedResourcePointer<juce::ScopedJuceInitialiser_GUI> fJuceInitialiser;
-#endif
-
     char* fStateChunk;
 
     // -------------------------------------------------------------------
@@ -1326,6 +1330,9 @@ const AEffect* VSTPluginMainInit(AEffect* const effect)
 #endif
 
     return effect;
+
+    // may be unused
+    (void)kBaseUniqueID;
 }
 
 // -----------------------------------------------------------------------
