@@ -72,10 +72,10 @@ bool waitForAsyncObject(const HANDLE object, const HANDLE process = INVALID_HAND
     {
         if (process != INVALID_HANDLE_VALUE)
         {
-            switch (WaitForSingleObject(process, 0))
+            switch (::WaitForSingleObject(process, 0))
             {
             case WAIT_OBJECT_0:
-            case -1:
+            case WAIT_FAILED:
                 carla_stderr("waitForAsyncObject process has stopped");
                 return false;
             }
@@ -245,9 +245,13 @@ bool startProcess(const char* const argv[], PROCESS_INFORMATION* const processIn
     carla_zeroStruct(startupInfo);
     startupInfo.cb = sizeof(startupInfo);
 
-    return ::CreateProcess(nullptr, const_cast<LPSTR>(command.toRawUTF8()),
-                           nullptr, nullptr, TRUE, CREATE_NO_WINDOW | CREATE_UNICODE_ENVIRONMENT,
-                           nullptr, nullptr, &startupInfo, processInfo) != FALSE;
+    if (::CreateProcessA(nullptr, const_cast<LPSTR>(command.toRawUTF8()),
+                         nullptr, nullptr, TRUE, CREATE_NO_WINDOW | CREATE_UNICODE_ENVIRONMENT,
+                         nullptr, nullptr, &startupInfo, processInfo) != FALSE)
+        return true;
+
+    carla_stderr2("startProcess failed, error was: %u", ::GetLastError());
+    return false;
 }
 
 static inline
@@ -436,7 +440,7 @@ bool waitForProcessToStop(const HANDLE process, const uint32_t timeOutMillisecon
         switch (::WaitForSingleObject(process, 0))
         {
         case WAIT_OBJECT_0:
-        case -1:
+        case WAIT_FAILED:
             return true;
         }
 
@@ -1209,9 +1213,15 @@ const char* CarlaPipeCommon::_readline(const bool allocReturn, const uint16_t si
     #endif
             } CARLA_SAFE_EXCEPTION_BREAK("CarlaPipeCommon::readline() - read");
 
-            if (ret != 1 || c == '\n')
+            if (ret != 1)
                 break;
 
+            if (c == '\n')
+            {
+                *ptr = '\0';
+                break;
+            }
+
             if (c == '\r')
                 c = '\n';
 
@@ -1604,7 +1614,6 @@ bool CarlaPipeServer::startPipeServer(const char* const filename,
         pData->processInfo.hThread  = INVALID_HANDLE_VALUE;
         try { ::CloseHandle(pipe1); } CARLA_SAFE_EXCEPTION("CloseHandle(pipe1)");
         try { ::CloseHandle(pipe2); } CARLA_SAFE_EXCEPTION("CloseHandle(pipe2)");
-        fail("startProcess() failed");
         return false;
     }
 
