@@ -1,6 +1,6 @@
 /*
  * Carla common utils
- * Copyright (C) 2011-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -124,9 +124,9 @@ void carla_debug(const char* const fmt, ...) noexcept
         {
             std::vfprintf(output, fmt, args);
             std::fprintf(output, "\n");
-            std::fflush(output);
         }
 
+        std::fflush(output);
         ::va_end(args);
     } CARLA_CATCH_UNWIND catch (...) {}
 }
@@ -145,7 +145,9 @@ void carla_stdout(const char* const fmt, ...) noexcept
         ::va_start(args, fmt);
         std::vfprintf(output, fmt, args);
         std::fprintf(output, "\n");
+#ifndef DEBUG
         if (output != stdout)
+#endif
             std::fflush(output);
         ::va_end(args);
     } CARLA_CATCH_UNWIND catch (...) {}
@@ -164,7 +166,9 @@ void carla_stderr(const char* const fmt, ...) noexcept
         ::va_start(args, fmt);
         std::vfprintf(output, fmt, args);
         std::fprintf(output, "\n");
+#ifndef DEBUG
         if (output != stderr)
+#endif
             std::fflush(output);
         ::va_end(args);
     } CARLA_CATCH_UNWIND catch (...) {}
@@ -192,9 +196,9 @@ void carla_stderr2(const char* const fmt, ...) noexcept
         {
             std::vfprintf(output, fmt, args);
             std::fprintf(output, "\n");
-            std::fflush(output);
         }
 
+        std::fflush(output);
         ::va_end(args);
     } CARLA_CATCH_UNWIND catch (...) {}
 }
@@ -251,6 +255,16 @@ void carla_safe_assert_uint2(const char* const assertion, const char* const file
     carla_stderr2("Carla assertion failure: \"%s\" in file %s, line %i, v1 %u, v2 %u", assertion, file, line, v1, v2);
 }
 
+/*
+ * Print a safe assertion error message, with a custom error message.
+ */
+static inline
+void carla_custom_safe_assert(const char* const message,
+                              const char* const assertion, const char* const file, const int line) noexcept
+{
+    carla_stderr2("Carla assertion failure: %s, condition \"%s\" in file %s, line %i", message, assertion, file, line);
+}
+
 // --------------------------------------------------------------------------------------------------------------------
 // carla_safe_exception*
 
